/**********************************************************************
** Copyright (C) 2002 Edgar Hindemith.  All rights reserved.
**
** email: ed@dom.de
**
** This file is part of the qpPhoto picture editor.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/
#include <qcolor.h>
#include <qlabel.h>
#include <qspinbox.h>
#include "colorPicker.h"
#include "colorDlg.h"

colorDlg::colorDlg(QWidget* parent, const char* name, bool modal, WFlags fl)
:ColorDialog(parent, name, modal, fl)
{
    selected.setRgb(192, 192, 192);
    red = 192;
    green = 192;
    blue = 192;
    connect( picker, SIGNAL(colorChanged()), this, SLOT(colorChanged()) );
    connect( rValue, SIGNAL(valueChanged(int)), this, SLOT(rChanged(int)) );
    connect( gValue, SIGNAL(valueChanged(int)), this, SLOT(gChanged(int)) );
    connect( bValue, SIGNAL(valueChanged(int)), this, SLOT(bChanged(int)) );
}

colorDlg::~colorDlg()
{
}

void colorDlg::setType(bool gray)
{
    picker->setType(gray);
    rValue->setEnabled(false);
    gValue->setEnabled(false);
    bValue->setEnabled(false);
    setFixedHeight(124);
}

void colorDlg::setTypeRGB()
{
    picker->setTypeRGB();
}


void colorDlg::setColor()
{
    QString hexCol;
    hexCol.sprintf("%02x%02x%02x", selected.red(), selected.green(), selected.blue());
    ColorHex->setText(hexCol);
    ShowColor->setBackgroundColor(selected);
}

void colorDlg::setColor(QColor color)
{
    selected = color;
    /*QString hexCol;
    hexCol.sprintf("%02x%02x%02x", selected.red(), selected.green(), selected.blue());
    ColorHex->setText(hexCol);
    ShowColor->setBackgroundColor(selected);
    */
    red = selected.red();
    green = selected.green();
    blue = selected.blue();
    rValue->setValue(red);
    gValue->setValue(green);
    bValue->setValue(blue);
    setColor();
}

void colorDlg::colorChanged()
{
    selected = picker->getColor();
    red = selected.red();
    green = selected.green();
    blue = selected.blue();
    rValue->setValue(red);
    gValue->setValue(green);
    bValue->setValue(blue);
    setColor();
}

void colorDlg::rChanged(int r)
{
    if(r != red){
        red = r;
        selected.setRgb(r, green, blue);
        setColor();
    }
}

void colorDlg::gChanged(int g)
{
    if(g != green){
        green = g;
        selected.setRgb(red, g, blue);
        setColor();
    }
}

void colorDlg::bChanged(int b)
{
    if(b != blue){
        blue = b;
        selected.setRgb(red, green, b);
        setColor();
    }
}
