/**********************************************************************
** Copyright (C) 2002 Edgar Hindemith.  All rights reserved.
**
** email: ed@dom.de
**
** This file is part of the qpPhoto picture editor.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/
#include <qcolor.h>
#include <qpixmap.h>
#include "colorPicker.h"

colorPicker::colorPicker(QWidget* parent, const char* name, WFlags fl)
:QLabel(parent, name, fl)
{
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 144, 96 ) );
    setMaximumSize( QSize( 144, 96 ) );
    setBaseSize( QSize( 144, 96 ) );
    grayimage = QImage(144, 96, 8);
    indexed = QImage(144,96,8);
    rgbImage = QImage(144, 96, 32);
    selected.setRgb(192, 192, 192);
    mouse_pressed = false;
    setMouseTracking(true);
    isRGB = false;
    isGray = true;
    setType(false);
}

colorPicker::~colorPicker()
{
}

void colorPicker::setType(bool gray)
{
    if(gray != isGray){
        QPixmap image0(144, 96);
        isGray = gray;
        if(gray){
            grayimage.setNumColors(256);
            for(int n = 0; n < 256; n++){
                int x = (n % 16);
                int y = (n / 16);
                grayimage.setColor(n, qRgb(n,n,n));
                for(int i = 0; i < 6; i++){
                    for(int k = 0; k < 9; k++){
                        grayimage.setPixel(x*9+k, y*6+i, n);
                    }
                }
            }
            image0.convertFromImage(grayimage);
        } else {
            indexed.setNumColors(216);
            for(int n = 0; n < 216; n++){
                int x = (n % 18) * 8;
                int y = (n / 18) * 8;
                if(y < 48){
                    int r = (x/48) * 51;
                    int g = (5-(y>>3)) * 51;
                    int b = ((x%48)>>3) * 51;
                    indexed.setColor(n, qRgb(r, g, b));
                } else {
                    int r = ((x + 144)/48) * 51;
                    int g = ((y - 48)>>3) * 51;
                    int b = (((x + 144)%48)>>3) * 51;
                    indexed.setColor(n, qRgb(r, g, b));
                }
                for(int i = 0; i < 8; i++){
                    for(int k = 0; k < 8; k++){
                        indexed.setPixel(x+k, y+i, n);
                    }
                }
            }
            image0.convertFromImage(indexed);
        }
        setPixmap( image0 );
    }
}

void colorPicker::setTypeRGB()
{
    for(int i = 0; i < 96; i++){
        for(int k = 0; k < 144; k++){
            int h = 0;
            int s = 0;
            int v = 0;
            if(i < 48)
                v = (i * 256)/48;
            else
                v = 255;
            if(i < 48)
                s = (i * 256)/48;
            else
                s = 255 - ((i - 48) * 256)/48;
            h = (k * 360)/144;
            QColor c;
            c.setHsv(h, s, v);
            rgbImage.setPixel(k, i, c.rgb());
        }
    }
    isRGB = true;
    QPixmap image0(144, 96);
    image0.convertFromImage(rgbImage);
    setPixmap( image0 );
}

void colorPicker::mouseMoveEvent(QMouseEvent* mev)
{
    mouse_x = mev->x();
    mouse_y = mev->y();
    if(mouse_pressed){
        calculateColor();
    }
    emit mouseMoved();
}

void colorPicker::mousePressEvent(QMouseEvent* mev)
{
    mouse_x = mev->x();
    mouse_y = mev->y();
    mouse_pressed = true;
    calculateColor();
    emit mousePressed();
}

void colorPicker::mouseReleaseEvent(QMouseEvent* mev)
{
    mouse_pressed = false;
    emit mousePressed();
}

void colorPicker::calculateColor()
{
    QColor color;
    if(mouse_x >= 0 && mouse_x < 144 && mouse_y >= 0 && mouse_y < 96){
        int x = mouse_x;
        int y = mouse_y;
        if(isGray){
            color.setRgb(grayimage.pixel(x, y));
        } else if(isRGB){
            color.setRgb(rgbImage.pixel(x, y));
        } else {
            color.setRgb(indexed.pixel(x, y));
        }
    }
    if(color != selected){
        selected = color;
        emit colorChanged();
    }
}
