/**********************************************************************
** Copyright (C) 2002 Edgar Hindemith.  All rights reserved.
**
** email: ed@dom.de
**
** This file is part of the qpPhoto picture editor.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/
#ifndef DRAWINCLUDED
#define DRAWINCLUDED

#include "drawwidget.h"
#include "drawview.h"

#include <qfont.h>
#include <qbrush.h>
#include <qpen.h>
#include <qpixmap.h>

class QColor;
class QString;
class QStringList;

class Draw : public DrawWidget
{
    Q_OBJECT
public:
    Draw( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~Draw();
    void init();

    enum DrawTool
    {
        dtNull,
        dtCursor,
        dtBrush,
        dtLine,
        dtPolygon,
        dtSelect,
        dtText,
        dtFill,
        dtPicker,
        dtErazor
    };

    enum Shapes
    {
        shRect,
        shFilledRect,
        shCircle,
        shFilledCircle,
        shEllipse,
        shFilledEllipse
    };

    enum BrushStyle
    {
        bsSquare,
        bsCircle3,
        bsCircle5,
        bsCircle7,
        bsCircle9,
        bsCircle11,
        bsCircle13,
        bsCircle7f,
        bsCircle9f,
        bsCircle11f,
        bsCircle13f
    };

    enum FillStyle
    {
        fillSolid,
        fillHGradient,
        fillVGradient,
        fillArea
    };

    enum SelectType
    {
        selRect,
        selCircle,
        selEllipse,
        selArea
    };


public slots:
    void selectFgColor();
    void selectBgColor();
    void setColor(QColor);
    void maskChanged();
    void pbCursorPressed(){ emit changeTool(dtCursor); };
    void pbBrushPressed(){ emit changeTool(dtBrush); };
    void pbLinePressed(){ emit changeTool(dtLine); };
    void pbPolygonPressed(){ emit changeTool(dtPolygon); };
    void pbSelectPressed(){ emit changeTool(dtSelect); };
    void pbTextPressed(){ emit changeTool(dtText); };
    void pbFillPressed(){ emit changeTool(dtFill); };
    void pbPickerPressed(){ emit changeTool(dtPicker); };
    void pbErazorPressed(){ emit changeTool(dtErazor); };

    void boldPressed();
    void italicPressed();
    void ulinePressed();
    void selChanged(int);
    void sizeChanged(int);
    void opChanged(int);

    void toolChanged(int tool);

    void statusChanged();
    void colorStateChanged(int, bool);

signals:
    void changeTool(int);

protected:
    void hideProps(int tool);
    void showProps(int tool);
    void setBrush();
    void setErazor();

public:
    DrawView *dv;

protected:
    int currentTool;
    int currentShape;
    int currentBrush;
    int currentErazorBrush;
    int currentPenStyle;
    int currentOp;
    int currentLineCap;
    int currentPicker;

    int brushSize;
    int erazorSize;
    bool bold;
    bool italic;
    bool underline;
    int depth;
    bool gray;
    /*QFont font;
    QBrush brush;
    QPen pen;
    QColor fgColor;
    QColor bgColor;
    QPixmap brushSprite;
    QPixmap erazorSprite;*/

    QString typeLabel;
    QString sizeLabel;
    QStringList availFonts;
};

#endif // DRAWINCLUDED

