/**********************************************************************
** Copyright (C) 2002 Edgar Hindemith.  All rights reserved.
**
** email: ed@dom.de
**
** This file is part of the qpPhoto picture editor.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/
#include "drawview.h"
#include "draw.h"
#include "TextInput.h"
#include "NewPic.h"

#include <qpaintdevice.h>
#include <qmultilineedit.h>
#include <qspinbox.h>
#include <qapplication.h>

#include <qbitmap.h>
#include <qtimer.h>
#include <qstring.h>
#include <math.h>

//static const char fmtPNG[]={'P','N','G',0};
static const char* fmtPNG={"PNG"};
static const int nzoomfactors = 7;
static const int zoomfactors[nzoomfactors] = {1,2,3,4,8,16,32}; // 25% 50% 75% 100% 200% 400% 800%
#define ZOOMINDEXNORMAL 3

DrawView::DrawView(QWidget* parent = 0, const char* name = 0, WFlags fl = 0)
    :QScrollView(parent, name, fl)
{
    viewX = 0;
    viewY = 0;
    mouse_x = 0;
    mouse_y = 0;
    imgWidth = 0;
    imgHeight = 0;
    depth = 32;
    gray = false;
    pasting = false;
    mouse_pressed = false;
    selection_hidden = false;
    scToggle = false;
    undoEnabled = true;
    setMouseTracking(true);
    currentShape = Draw::shRect;
    currentTool = Draw::dtNull;
    currentFill = Draw::fillSolid;
    currentLineCap = 0;
    currentSelType = Draw::selRect;
    currentSelOp = 0;
    currentRO = roSTD;
    currentROC = rocALWAYS;
    QTimer *timer = new QTimer(this);
    connect( timer, SIGNAL(timeout()), this, SLOT(timeoutUpdate()) );
    timer->start(1000);
}

DrawView::~DrawView()
{
}

void DrawView::init(int w, int h, const QColor bg, const QColor fg)
{
    if(w < 0 || h < 0){
        NewPic* np = new NewPic(this, "new picture", true);
        int result = np->exec();
        imgWidth = 200, imgHeight = 200;
        if(result == QDialog::Accepted){
            imgWidth = np->sbWidth->value();
            imgHeight = np->sbHeight->value();
        }
        delete np;
    } else {
        imgWidth = w;
        imgHeight = h;
    }
    depth = 32;
    gray = false;
    bgColor = bg;
    fgColor = fg;
    trackerOrigin_x = -1;
    trackerOrigin_y = -1;
    setBrushImage(1);
    setErazorImage(1);
    zoomIndex = ZOOMINDEXNORMAL;
    isScaled = false;
    scaledWidth = imgWidth;
    scaledHeight = imgHeight;
    undoState = usCantDo;
    undoType = utImage;
    imageFormat = fmtPNG;
    selection = QRegion();
    clipRegion = QRegion();
    undoRegion = QRegion();
    pmScaled.resize(0, 0);
    clipBits.resize(0, 0);
    pmUndo.resize(imgWidth, imgHeight);
    graphic.resize(imgWidth, imgHeight);
    graphic.fill(bgColor);
    emit selectionChanged();
    emit zoomStateChanged(zsCanZoom);
    emit undoStateChanged(undoState);
    emit clipStateChanged();
    emit colorStateChanged(depth, gray);
    resizeContents(imgWidth, imgHeight);
    repaintContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
}

void DrawView::init(QString fileName, const QColor bg, const QColor fg)
{
    bgColor = bg;
    fgColor = fg;
    trackerOrigin_x = -1;
    trackerOrigin_y = -1;
    setBrushImage(1);
    setErazorImage(1);
    zoomIndex = ZOOMINDEXNORMAL;
    isScaled = false;
    scaledWidth = imgWidth;
    scaledHeight = imgHeight;
    undoState = usCantDo;
    undoType = utImage;
    selection = QRegion();
    clipRegion = QRegion();
    undoRegion = QRegion();
    pmScaled.resize(0, 0);
    clipBits.resize(0, 0);
    imageFormat = QImageIO::imageFormat(fileName);
    /*if(graphic.load(fileName, imageFormat, QPixmap::AvoidDither)){
        imgWidth = graphic.width();
        imgHeight = graphic.height();*/
    if(image.load(fileName, imageFormat)){
        imgWidth = image.width();
        imgHeight = image.height();
        gray = image.isGrayscale();
        depth = image.depth();
        graphic.convertFromImage(image, 0);
    } else {
        imgWidth = width();
        imgHeight = height();
        graphic.resize(imgWidth, imgHeight);
        graphic.fill(bgColor);
        depth = 32;
        gray = false;
    }
    if(undoEnabled)
        pmUndo.resize(imgWidth, imgHeight);
    scale();
    emit selectionChanged();
    emit zoomStateChanged(zsCanZoom);
    emit undoStateChanged(undoState);
    emit clipStateChanged();
    emit colorStateChanged(depth, gray);
}

void DrawView::init(QImage img)
{
    trackerOrigin_x = -1;
    trackerOrigin_y = -1;
    imgWidth = img.width(),
    imgHeight = img.height(),
    zoomIndex = ZOOMINDEXNORMAL;
    isScaled = false;
    scaledWidth = imgWidth;
    scaledHeight = imgHeight;
    undoState = usCantDo;
    undoType = utImage;
    selection = QRegion();
    clipRegion = QRegion();
    undoRegion = QRegion();
    pmScaled.resize(0, 0);
    clipBits.resize(0, 0);
    image = img;
    gray = image.isGrayscale();
    depth = image.depth();
    graphic.convertFromImage(image);
    if(undoEnabled)
        pmUndo.resize(imgWidth, imgHeight);
    emit selectionChanged();
    emit zoomStateChanged(zsCanZoom);
    emit undoStateChanged(undoState);
    emit clipStateChanged();
    emit colorStateChanged(depth, gray);
    resizeContents(imgWidth, imgHeight);
    updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
}

void DrawView::enableUndo(bool enUndo)
{
    if(undoEnabled != enUndo){
        if(enUndo)
            pmUndo.resize(imgWidth, imgHeight);
        else
            pmScaled.resize(0, 0);
        undoEnabled = enUndo;
        undoState = usCantDo;
        emit undoStateChanged(undoState);
    }
}

void DrawView::resizeContents(int w, int h)
{
    int ws = verticalScrollBar()->width();
    int hs = horizontalScrollBar()->height();
    if(w + ws < width())
        setHScrollBarMode(AlwaysOff);
    else
        setHScrollBarMode(AlwaysOn);
    if(h + hs < height())
        setVScrollBarMode(AlwaysOff);
    else
        setVScrollBarMode(AlwaysOn);

    QScrollView::resizeContents(w, h);
}

void DrawView::setBrushImage(int size)
{
    brushSprite = QPixmap(size, size);
    brushSprite.fill(fgColor);
    brushImage = brushSprite.convertToImage();
    brushWidth = size;
    brushHeight = size;
    brushMask = QImage();
    brushSolid = true;
}

void DrawView::setBrushImage(const char** mask)
{
    QPixmap mm;
    mm = QPixmap(mask);
    setBrushImage(mm);
}

void DrawView::setBrushImage(const QPixmap& mask)
{
    brushWidth = mask.width();
    brushHeight = mask.height();
    brushSprite = QPixmap(brushWidth, brushHeight);

    brushMask = mask.convertToImage();
    QPainter painter;
    painter.begin(&brushSprite);

    for(int i = 0; i < brushWidth; i++){
        for(int k = 0; k < brushHeight; k++){
            painter.setPen((QColor(brushMask.pixel(i,k)) == black)?fgColor:white);
            painter.drawPoint(i, k);
        }
    }
    painter.end();
    brushImage = brushSprite.convertToImage();
    brushSolid = true;
}

void DrawView::setBrushImage(const QPixmap& image, const QPixmap& mask)
{
    brushWidth = mask.width();
    brushHeight = mask.height();
    brushSprite = QPixmap(brushWidth, brushHeight);

    brushMask = mask.convertToImage();
    QImage img = image.convertToImage();
    brushImage = brushSprite.convertToImage();
    QPainter painter;
    painter.begin(&brushSprite);

    for(int i = 0; i < brushWidth; i++){
        for(int k = 0; k < brushHeight; k++){
            QColor pc = QColor( (QColor(brushMask.pixel(i,k)) == black)?((QColor(img.pixel(i,k)) == black)?fgColor:pixelOp(img.pixel(i,k),fgColor.rgb(),roMASK)):white );
            painter.setPen(pc);
            painter.drawPoint(i, k);
            brushImage.setPixel(i, k, pc.rgb());
        }
    }
    painter.end();
    brushSolid = false;
}

void DrawView::setErazorImage(int size)
{
    erazorSprite = QPixmap(size, size);
    erazorSprite.fill(bgColor);
    erazorImage = erazorSprite.convertToImage();
    erazorWidth = size;
    erazorHeight = size;
    erazorMask = QImage();
    erazorSolid = true;
}

void DrawView::setErazorImage(const QPixmap& mask)
{
    erazorWidth = mask.width();
    erazorHeight = mask.height();
    erazorSprite = QPixmap(erazorWidth, erazorHeight);

    erazorMask = mask.convertToImage();
    QPainter painter;
    painter.begin(&erazorSprite);

    for(int i = 0; i < erazorWidth; i++){
        for(int k = 0; k < erazorHeight; k++){
            painter.setPen((QColor(erazorMask.pixel(i,k)) == black)?bgColor:black);
            painter.drawPoint(i, k);
        }
    }
    painter.end();
    erazorImage = erazorSprite.convertToImage();
    erazorSolid = true;
}

void DrawView::setErazorImage(const QPixmap& image, const QPixmap& mask)
{
    erazorWidth = mask.width();
    erazorHeight = mask.height();
    erazorSprite = QPixmap(erazorWidth, erazorHeight);

    erazorMask = mask.convertToImage();
    QImage img = image.convertToImage();
    erazorImage = erazorSprite.convertToImage();
    QPainter painter;
    painter.begin(&erazorSprite);

    for(int i = 0; i < erazorWidth; i++){
        for(int k = 0; k < erazorHeight; k++){
            QColor pc = QColor( (QColor(erazorMask.pixel(i,k)) == black)?((QColor(img.pixel(i,k)) == black)?bgColor:pixelOp(img.pixel(i,k),bgColor.rgb(),roMASK)):black );
            painter.setPen(pc);
            painter.drawPoint(i, k);
            erazorImage.setPixel(i, k, pc.rgb());
        }
    }
    painter.end();
    erazorSolid = false;
}

QColor DrawView::pixelOp(QRgb src, QRgb dst, int rop)
{
    if(rop == roSTD){
        if(currentTool == Draw::dtBrush){
            if(!brushSolid && !(QColor(src) == fgColor)){
                return QColor((src & dst));
            }
            return fgColor;
        } else if(currentTool == Draw::dtErazor){
            if(!erazorSolid && !(QColor(src) == bgColor)){
                return QColor((src & dst));
            }
            return bgColor;
        }
        return QColor(src);
    } else if(rop == roAND){
        return QColor((src & dst));
    } else if(rop == roOR){
        return QColor((src | dst));
    } else if(rop == roMASK){
        int h,s,v;
        QColor c1 = QColor(src);
        QColor c2 = QColor(dst);
        c2.hsv(&h,&s,&v);
        int g = (c1.red()*11 + c1.green()*16 + c1.blue()*5)/32;
        QColor c3;
        c3.setHsv(h, ((s - g) < 0)?0:(s - g), ((g + v) > 255)?255:(g + v));
        return QColor(c3);
    } else if(rop == roMIX){
        int h1,h2,s1,s2,v1,v2;
        QColor c1 = QColor(src);
        QColor c2 = QColor(dst);
        c1.hsv(&h1,&s1,&v1);
        c2.hsv(&h2,&s2,&v2);
        QColor c3;
        c3.setHsv((h1+h2)>>1, (s1+s2)>>1, (v1+v2)>>1);
        return QColor(c3);
    } else if(rop == roCP){
        return QColor(src);
    } else if(rop == roADD){
        QColor srcPixelColor = QColor(src);
        QColor dstPixelColor = QColor(dst);
        int r = (srcPixelColor.red() + dstPixelColor.red());
        int g = (srcPixelColor.green() + dstPixelColor.green());
        int b = (srcPixelColor.blue() + dstPixelColor.blue());
        return QColor((r > 255)?255:r, (g > 255)?255:g, (b > 255)?255:b);
    } else if(rop == roSUB){
        QColor srcPixelColor = QColor(src);
        QColor dstPixelColor = QColor(dst);
        int r = (srcPixelColor.red() - dstPixelColor.red());
        int g = (srcPixelColor.green() - dstPixelColor.green());
        int b = (srcPixelColor.blue() - dstPixelColor.blue());
        return QColor((r < 0)?0:r, (g < 0)?0:g, (b < 0)?0:b);
    } else if(rop == roLIGHT){
        QColor srcPixelColor = QColor(src);
        QColor dstPixelColor = QColor(dst);
        int r = (((srcPixelColor.red()*dstPixelColor.red())>>8) + dstPixelColor.red());
        int g = (((srcPixelColor.green()*dstPixelColor.green())>>8) + dstPixelColor.green());
        int b = (((srcPixelColor.blue()*dstPixelColor.blue())>>8) + dstPixelColor.blue());
        return QColor((r > 255)?255:r, (g > 255)?255:g, (b > 255)?255:b);
    } else if(rop == roDARK){
        QColor srcPixelColor = QColor(src);
        QColor dstPixelColor = QColor(dst);
        int r = (dstPixelColor.red() - ((srcPixelColor.red()*dstPixelColor.red())>>8));
        int g = (dstPixelColor.green() - ((srcPixelColor.green()*dstPixelColor.green())>>8));
        int b = (dstPixelColor.blue() - ((srcPixelColor.blue()*dstPixelColor.blue())>>8));
        return QColor((r < 0)?0:r, (g < 0)?0:g, (b < 0)?0:b);
    } else if(rop == roAVG){
        QColor srcPixelColor = QColor(src);
        QColor dstPixelColor = QColor(dst);
        int r = ((srcPixelColor.red() + dstPixelColor.red())>>1);
        int g = ((srcPixelColor.green() + dstPixelColor.green())>>1);
        int b = ((srcPixelColor.blue() + dstPixelColor.blue())>>1);
        return QColor(r, g, b);
    }
    return white;
}

QColor DrawView::pixelColor(int x, int y, QImage& dst, QImage& src, QImage& mask, int rop, int ropc, bool useMask)
{
    if(!mask.isNull()){
        bool m = (bool)(QColor(mask.pixel(x,y)) == black);
        if(!m && useMask){
            return QColor(dst.pixel(x,y));
        }
    }
    if(ropc == rocALWAYS){
        return pixelOp(src.pixel(x,y), dst.pixel(x,y), rop);
    } else if(ropc == rocIFLT){
        QColor srcPixelColor = QColor(src.pixel(x,y));
        QColor dstPixelColor = QColor(dst.pixel(x,y));
        bool r = (bool)(srcPixelColor.red() < dstPixelColor.red());
        bool g = (bool)(srcPixelColor.green() < dstPixelColor.green());
        bool b = (bool)(srcPixelColor.blue() < dstPixelColor.blue());
        if(r && g && b)return pixelOp(src.pixel(x,y), dst.pixel(x,y), rop);
        return dstPixelColor;
    } else if(ropc == rocIFGT){
        QColor srcPixelColor = QColor(src.pixel(x,y));
        QColor dstPixelColor = QColor(dst.pixel(x,y));
        bool r = (bool)(srcPixelColor.red() > dstPixelColor.red());
        bool g = (bool)(srcPixelColor.green() > dstPixelColor.green());
        bool b = (bool)(srcPixelColor.blue() > dstPixelColor.blue());
        if(r && g && b)return pixelOp(src.pixel(x,y), dst.pixel(x,y), rop);
        return dstPixelColor;
    }
    return black;
}

void DrawView::blit(int x, int y, int pt)
{
    QPainter painter;
    if(currentTool == Draw::dtBrush){
        int xo = brushWidth/2;
        int yo = brushHeight/2;
        QPixmap dstImage;
        dstImage.resize( brushWidth, brushHeight );
        bitBlt(&dstImage, 0, 0, &graphic, x-xo, y-yo, brushWidth, brushHeight, CopyROP);
        QImage img = dstImage.convertToImage();
        painter.begin((isScaled)?&pmScaled:&graphic);
        if(!selection.isEmpty())
            painter.setClipRegion(selection);
        int factor = isScaled?zoomfactors[zoomIndex]:4;
        xo = xo * factor / 4;
        yo = yo * factor / 4;
        if(pt == ptMiddle){
            for(int i = 0; i < brushWidth * factor / 4; i++){
                painter.setPen(pixelColor(i * 4 / factor, yo * 4 / factor, img, brushImage, brushMask, currentRO, currentROC));
                painter.drawPoint(x-xo+i, y);
            }
            for(int k = 0; k < brushHeight * factor / 4; k++){
                painter.setPen(pixelColor(xo * 4 / factor, k * 4 / factor, img, brushImage, brushMask, currentRO, currentROC));
                painter.drawPoint(x, y-yo+k);
            }
        } else {
            int start_x = 0;
            int start_y = 0;
            int end_x = brushWidth * factor / 4;
            int end_y = brushHeight * factor / 4;
            switch(pt){
                case ptEnd1:
                    start_x = xo;
                    break;
                case ptEnd2:
                    start_y = yo;
                    break;
                case ptEnd3:
                    end_x = xo;
                    break;
                case ptEnd4:
                    end_y = yo;
                    break;
            }
            for(int i = start_x; i < end_x; i++){
                for(int k = start_y; k < end_y; k++){
                    painter.setPen(pixelColor(i * 4 / factor, k * 4 / factor, img, brushImage, brushMask, currentRO, currentROC));
                    painter.drawPoint(x-xo+i, y-yo+k);
                }
            }
         }
         painter.end();
    } else if(currentTool == Draw::dtErazor){
        int xo = erazorWidth/2;
        int yo = erazorHeight/2;
        QPixmap dstImage;
        dstImage.resize( erazorWidth, erazorHeight );
        bitBlt(&dstImage, 0, 0, &graphic, x-xo, y-yo, erazorWidth, erazorHeight, CopyROP);
        QImage img = dstImage.convertToImage();
        painter.begin((isScaled)?&pmScaled:&graphic);
        int factor = isScaled?zoomfactors[zoomIndex]:4;
        xo = xo * factor / 4;
        yo = yo * factor / 4;
        if(pt == ptMiddle){
            for(int i = 0; i < erazorWidth * factor / 4; i++){
                painter.setPen(pixelColor(i * 4 / factor, yo * 4 / factor, img, erazorImage, erazorMask, currentRO, currentROC));
                painter.drawPoint(x-xo+i, y);
            }
            for(int k = 0; k < erazorHeight * factor / 4; k++){
                painter.setPen(pixelColor(xo * 4 / factor, k * 4 / factor, img, erazorImage, erazorMask, currentRO, currentROC));
                painter.drawPoint(x, y-yo+k);
            }
        } else {
            int start_x = 0;
            int start_y = 0;
            int end_x = erazorWidth * factor / 4;
            int end_y = erazorHeight * factor / 4;
            switch(pt){
                case ptEnd1:
                    start_x = xo;
                    break;
                case ptEnd2:
                    start_y = yo;
                    break;
                case ptEnd3:
                    end_x = xo;
                    break;
                case ptEnd4:
                    end_y = yo;
                    break;
            }
            for(int i = start_x; i < end_x; i++){
                for(int k = start_y; k < end_y; k++){
                    painter.setPen(pixelColor(i * 4 / factor, k * 4 / factor, img, erazorImage, erazorMask, currentRO, currentROC));
                    painter.drawPoint(x-xo+i, y-yo+k);
                }
            }
         }
         painter.end();
    }
}


void DrawView::drawPoint(int x, int y)
{
    blit(x,y);
}

void DrawView::drawEndPoint(int x, int y)
{
    blit(x,y);
    int temp_x = last_x;
    int temp_y = last_y;
    last_x = x;
    last_y = y;
    drawLineTo(temp_x, temp_y);
}

void DrawView::drawLineTo(int x, int y)
{
    int dx = x - last_x;
    int dy = y - last_y;
    uint dxa = (dx >= 0)?dx:(-1*dx);
    uint dya = (dy >= 0)?dy:(-1*dy);
    int count = (dxa > dya)?dxa:dya;
    int dir = dirNull;
    if(dx > 0 && dy > 0){
        dir = dir1_30;
    } else if(dx > 0 && dy == 0){
        dir = dir3;
    } else if(dx > 0 && dy < 0){
        dir = dir4_30;
    } else if(dx == 0 && dy < 0){
        dir = dir6;
    } else if(dx < 0 && dy < 0){
        dir = dir7_30;
    } else if(dx < 0 && dy == 0){
        dir = dir9;
    } else if(dx < 0 && dy > 0){
        dir = dir10_30;
    } else if(dx == 0 && dy > 0){
        dir = dir12;
    }
    if(dir != direction){
        switch(direction){
            case dir1_30:
            case dir3:
            case dir4_30:
                blit(last_x, last_y, ptEnd1);
                break;
            case dir6:
                blit(last_x, last_y, ptEnd2);
                break;
            case dir7_30:
            case dir9:
            case dir10_30:
                blit(last_x, last_y, ptEnd3);
                break;
            case dir12:
                blit(last_x, last_y, ptEnd4);
                break;
        }
        direction = dir;
    }
    for(int i = 0; i < count; i++){
        blit(last_x + (i*dx)/count, last_y + (i*dy)/count, ptMiddle);
    }
}

void DrawView::drawTracker(int x, int y, int type, bool draw)
{
    QPainter painter;
    if(trackerOrigin_x >= 0 && trackerOrigin_y >= 0){
        painter.begin((isScaled)?&pmScaled:&graphic);
        QPen p = QPen(white, 1);
        painter.setPen(p);
        painter.setRasterOp(XorROP);
        int w = last_x - trackerOrigin_x;
        int h = last_y - trackerOrigin_y;
        /* erease old */
        switch(type){
            case trLine:
            painter.drawLine(trackerOrigin_x, trackerOrigin_y, last_x, last_y);
            break;
            case trHLine:
            painter.drawLine(trackerOrigin_x, trackerOrigin_y, last_x, trackerOrigin_y);
            break;
            case trVLine:
            painter.drawLine(trackerOrigin_x, trackerOrigin_y, trackerOrigin_x, last_y);
            break;
            case trRect:
            painter.drawRect(trackerOrigin_x, trackerOrigin_y, last_x - trackerOrigin_x, last_y - trackerOrigin_y);
            break;
            case trCircle:
            p.setStyle(DotLine);
            painter.setPen(p);
            if(w < h)
                painter.drawArc(trackerOrigin_x, trackerOrigin_y + h/2 - w/2, w, w, 0, 5760);
            else
                painter.drawArc(trackerOrigin_x + w/2 - h/2, trackerOrigin_y, h, h, 0, 5760);
            break;
            case trEllipse:
            p.setStyle(DotLine);
            painter.setPen(p);
            painter.drawEllipse(trackerOrigin_x, trackerOrigin_y, last_x - trackerOrigin_x, last_y - trackerOrigin_y);
            break;
            case trMove:
            painter.drawRect(trackerMoveRect.x() - trackerOrigin_x + last_x, trackerMoveRect.y() - trackerOrigin_y + last_y, trackerMoveRect.width(), trackerMoveRect.height());
            break;
        }
        w = x - trackerOrigin_x;
        h = y - trackerOrigin_y;
        if(draw){
            /* paint new */
            switch(type){
                case trLine:
                painter.drawLine(trackerOrigin_x, trackerOrigin_y, x, y);
                break;
                case trHLine:
                painter.drawLine(trackerOrigin_x, trackerOrigin_y, x, trackerOrigin_y);
                break;
                case trVLine:
                painter.drawLine(trackerOrigin_x, trackerOrigin_y, trackerOrigin_x, y);
                break;
                case trRect:
                painter.drawRect(trackerOrigin_x, trackerOrigin_y, w, h);
                break;
                case trCircle:
                p.setStyle(DotLine);
                painter.setPen(p);
                if(w < h)
                    painter.drawArc(trackerOrigin_x, trackerOrigin_y + h/2 - w/2, w, w, 0, 5760);
                else
                    painter.drawArc(trackerOrigin_x + w/2 - h/2, trackerOrigin_y, h, h, 0, 5760);
                break;
                case trEllipse:
                p.setStyle(DotLine);
                painter.setPen(p);
                painter.drawEllipse(trackerOrigin_x, trackerOrigin_y, w, h);
                break;
                case trMove:
                painter.drawRect(trackerMoveRect.x() - trackerOrigin_x + x, trackerMoveRect.y() - trackerOrigin_y + y, trackerMoveRect.width(), trackerMoveRect.height());
                break;
            }
        } else {
            if(type == trLine || type == trHLine || type == trVLine){
                painter.drawPoint(trackerOrigin_x, trackerOrigin_y);
            }
        }
        painter.end();
    }
}

void DrawView::startTracker(int x, int y)
{
    trackerOrigin_x = x;
    trackerOrigin_y = y;
    last_x = x;
    last_y = y;
    int factor = zoomfactors[zoomIndex];
    QPoint p = QPoint(x * 4 / factor, y * 4 / factor);
    switch(currentTool){
        case Draw::dtCursor:
            if(selection.contains(p)){
                trackerMoveRect = selection.boundingRect();
                if(isScaled){
                    trackerMoveRect.setLeft(trackerMoveRect.left() * factor / 4);
                    trackerMoveRect.setTop(trackerMoveRect.top() * factor / 4);
                    trackerMoveRect.setRight(trackerMoveRect.right() * factor / 4);
                    trackerMoveRect.setBottom(trackerMoveRect.bottom() * factor / 4);
                }
                selection_hidden = true;
                drawTracker(x, y, trMove, false);
            }
            break;
        case Draw::dtText:
            drawTracker(x, y);
            break;
        case Draw::dtBrush:
        case Draw::dtErazor:
            drawPoint(x, y);
            points.append(QPoint(x,y));
            break;
        case Draw::dtLine:
            drawTracker(x, y, trLine);
            break;
        case Draw::dtFill:
            switch(currentFill){
                case Draw::fillHGradient:
                drawTracker(x, y, trHLine);
                break;
                case Draw::fillVGradient:
                drawTracker(x, y, trVLine);
                break;
            }
            break;
        case Draw::dtSelect:
            switch(currentSelType){
                case Draw::selRect:
                    drawTracker(x, y);
                    break;
                case Draw::selCircle:
                    drawTracker(x, y, trCircle);
                    break;
                case Draw::selEllipse:
                    drawTracker(x, y, trEllipse);
                    break;
            }
            break;
        case Draw::dtPolygon:
            switch(currentShape){
                case Draw::shRect:
                case Draw::shFilledRect:
                    drawTracker(x, y);
                    break;
                case Draw::shCircle:
                case Draw::shFilledCircle:
                    drawTracker(x, y, trCircle);
                    break;
                case Draw::shEllipse:
                case Draw::shFilledEllipse:
                    drawTracker(x, y, trEllipse);
                    break;
            }
            break;
    }
}

void DrawView::track(int x, int y)
{
    if(!pasting){
        switch(currentTool){
            case Draw::dtText:
                drawTracker(x, y);
                break;
            case Draw::dtCursor:
                drawTracker(x, y, trMove);
                break;
            case Draw::dtBrush:
            case Draw::dtErazor:
                drawLineTo(x, y);
                points.append(QPoint(x,y));
                break;
            case Draw::dtLine:
                drawTracker(x, y, trLine);
                break;
            case Draw::dtFill:
                switch(currentFill){
                    case Draw::fillHGradient:
                    drawTracker(x, y, trHLine);
                    break;
                    case Draw::fillVGradient:
                    drawTracker(x, y, trVLine);
                    break;
                }
                break;
            case Draw::dtSelect:
                switch(currentSelType){
                    case Draw::selRect:
                        drawTracker(x, y);
                        break;
                    case Draw::selCircle:
                        drawTracker(x, y, trCircle);
                        break;
                    case Draw::selEllipse:
                        drawTracker(x, y, trEllipse);
                        break;
                }
                break;
            case Draw::dtPolygon:
                switch(currentShape){
                    case Draw::shRect:
                    case Draw::shFilledRect:
                        drawTracker(x, y);
                        break;
                    case Draw::shCircle:
                    case Draw::shFilledCircle:
                        drawTracker(x, y, trCircle);
                        break;
                    case Draw::shEllipse:
                    case Draw::shFilledEllipse:
                        drawTracker(x, y, trEllipse);
                        break;
                }
                break;
        }
    } else {
        drawTracker(x, y, trMove);
    }
    last_x = x;
    last_y = y;
}

void DrawView::endTracker(int x, int y)
{
    int i,c;
    bool oldScaled;
    bool undoSet = false;
    int factor = zoomfactors[zoomIndex];
    if(!pasting){
        switch(currentTool){
            case Draw::dtText:
                drawTracker(x, y, trRect, false);
                break;
            case Draw::dtCursor:
                drawTracker(x, y, trMove, false);
                break;
            case Draw::dtBrush:
            case Draw::dtErazor:
                setUndo();
                undoSet = true;
                oldScaled = isScaled;
                if(isScaled)isScaled = false;
                c = points.count();
                if(c > 0){
                    drawPoint(points[0].x()*4/factor, points[0].y()*4/factor);
                    last_x = points[0].x()*4/factor;
                    last_y = points[0].y()*4/factor;
                }
                for(i = 1; i < c; i++){
                    drawLineTo(points[i].x()*4/factor, points[i].y()*4/factor);
                    last_x = points[i].x()*4/factor;
                    last_y = points[i].y()*4/factor;
                }
                points.clear();
                drawEndPoint(x*4/factor, y*4/factor);
                isScaled = oldScaled;
                break;
            case Draw::dtLine:
                drawTracker(x, y, trLine, false);
                break;
            case Draw::dtFill:
                switch(currentFill){
                    case Draw::fillHGradient:
                    drawTracker(x, y, trHLine, false);
                    break;
                    case Draw::fillVGradient:
                    drawTracker(x, y, trVLine, false);
                    break;
                }
                break;
            case Draw::dtSelect:
                switch(currentSelType){
                    case Draw::selRect:
                        drawTracker(x, y, trRect, false);
                        break;
                    case Draw::selCircle:
                        drawTracker(x, y, trCircle, false);
                        break;
                    case Draw::selEllipse:
                        drawTracker(x, y, trEllipse, false);
                        break;
                }
                break;
            case Draw::dtPolygon:
                switch(currentShape){
                    case Draw::shRect:
                    case Draw::shFilledRect:
                        drawTracker(x, y, trRect, false);
                        break;
                    case Draw::shCircle:
                    case Draw::shFilledCircle:
                        drawTracker(x, y, trCircle, false);
                        break;
                    case Draw::shEllipse:
                    case Draw::shFilledEllipse:
                        drawTracker(x, y, trEllipse, false);
                        break;
                }
                break;
        }
        if(!undoSet)
            setUndo();
        if(isScaled){
            int factor = zoomfactors[zoomIndex];
            trackerOrigin_x = (trackerOrigin_x * 4)/factor;
            trackerOrigin_y = (trackerOrigin_y * 4)/factor;
            doDrawTool((x * 4)/factor, (y * 4)/factor);
            scale();
        } else {
            doDrawTool(x, y);
        }
    } else {
        drawTracker(x, y, trMove, false);
        int px = trackerMoveRect.x() - trackerOrigin_x + x;
        int py = trackerMoveRect.y() - trackerOrigin_y + y;
        setUndo();
        if(isScaled){
            int factor = zoomfactors[zoomIndex];
            paste((px * 4)/factor, (py * 4)/factor);
            scale();
        } else {
            paste(px, py);
        }
    }

    last_x = x;
    last_y = y;
    trackerOrigin_x = -1;
    trackerOrigin_y = -1;
}

void DrawView::drawLine(QPainter& painter, int x1, int y1, int x2, int y2)
{
    /*
        currentLineCap == 0 : --------
        currentLineCap == 1 : ------|>
        currentLineCap == 2 : <|----|>
        currentLineCap == 3 : |-------
        currentLineCap == 4 : |------|
        currentLineCap == 5 : |-----|>
    */
    if(currentLineCap != 0){
        int pw = painter.pen().width();
        int dx = (x2 - x1);
        int dy = (y2 - y1);
        double l = sqrt((dx*dx + dy*dy));
        double a = 0.0, a1 = 0.0, a2 = 0.0;
        if(dx >= 0 && dy < 0){
            a = acos(((double)dx/l));
            a1 = M_PI_2 - a;
            a2 = -1.0 * a;
        }else if(dx < 0 && dy < 0){
            a = asin(((double)dy/l));
            a1 = M_PI_2 - a;
            a2 = M_PI - a;
        }else if(dx < 0 && dy >= 0){
            a = acos((fabs((double)dx)/l));
            a1 = M_PI_2 - a;
            a2 = M_PI - a;
        }else if(dx >= 0 && dy >= 0){
            a = asin((fabs((double)dy)/l));
            a1 = M_PI_2 + a;
            a2 = a;
        }
        dx = (int)((double)pw * 3.5 * cos(a1));
        dy = (int)((double)pw * 3.5 * sin(a1));

        int x3 = x1, x4 = x2, y3 = y1, y4 = y2;

        if(currentLineCap == 2){
            // move start-point
            x3 += (int)((double)pw * 8.0 * cos(a2));
            y3 += (int)((double)pw * 8.0 * sin(a2));
            // draw arrow: <
            painter.drawLine(x3 + dx, y3 + dy, x1, y1);
            painter.drawLine(x1, y1, x3 - dx, y3 - dy);
        }

        if(currentLineCap == 1 || currentLineCap == 2 || currentLineCap == 5){
            // move end-point
            x4 -= (int)((double)pw * 8.0 * cos(a2));
            y4 -= (int)((double)pw * 8.0 * sin(a2));
            // draw arrow: >
            painter.drawLine(x4 + dx, y4 + dy, x2, y2);
            painter.drawLine(x2, y2, x4 - dx, y4 - dy);
        }

        if(currentLineCap != 1){
            // bar at start-point: |-----
            painter.drawLine(x3 + dx, y3 + dy, x3 - dx, y3 - dy);
        }

        if(currentLineCap != 3){
            // bar at end-point: -----|
            painter.drawLine(x4 + dx, y4 + dy, x4 - dx, y4 - dy);
        }

        painter.drawLine(x3, y3, x4, y4);
        //fprintf(stderr, "a = %d a1 = %d a2 = %d\n", (int)(180 * a / M_PI), (int)(180 * a1 / M_PI), (int)(180 * a2 / M_PI));
    } else {
        painter.drawLine(x1, y1, x2, y2);
    }
}

void DrawView::fill(QPainter& painter, int start, int end)
{
    if(currentFill == Draw::fillSolid){
        QBrush fillBrush = QBrush(fgColor);
        painter.fillRect(0, 0, imgWidth, imgHeight, fillBrush);
    } else if(currentFill == Draw::fillArea){
        int x = start;
        int y = end;
        QRegion fillRegion;
        findArea(x, y, getColorAt(x, y), fillRegion);
        QRegion oldSelection = selection;
        QRegion targetRegion = (selection.isEmpty())?fillRegion:selection.intersect(fillRegion);
        painter.setClipRegion(targetRegion);
        QBrush fillBrush = QBrush(fgColor);
        painter.fillRect(0, 0, imgWidth, imgHeight, fillBrush);
        selection = oldSelection;
        painter.setClipRegion(selection);
    } else if(currentFill == Draw::fillHGradient){
        int dr = (fgColor.red() > bgColor.red())?fgColor.red() - bgColor.red():bgColor.red() - fgColor.red();
        int dg = (fgColor.green() > bgColor.green())?fgColor.green() - bgColor.green():bgColor.green() - fgColor.green();
        int db = (fgColor.blue() > bgColor.blue())?fgColor.blue() - bgColor.blue():bgColor.blue() - fgColor.blue();
        if(start <= end){
            int dc = end - start;
            for(int i = 0; i < imgWidth; i++){
                QColor lineColor = fgColor;
                if(i >= start && i < end){
                    lineColor = QColor( (fgColor.red() > bgColor.red())?fgColor.red() - (dr*(i-start))/dc:fgColor.red() + (dr*(i-start))/dc,
                                        (fgColor.green() > bgColor.green())?fgColor.green() - (dg*(i-start))/dc:fgColor.green() + (dg*(i-start))/dc,
                                        (fgColor.blue() > bgColor.blue())?fgColor.blue() - (db*(i-start))/dc:fgColor.blue() + (db*(i-start))/dc);
                } else if(i >= end){
                    lineColor = bgColor;
                }
                QPen p = QPen(lineColor, 1);
                painter.setPen(p);
                painter.drawLine(i, 0, i, imgHeight);
            }
        } else {
            int dc = start - end;
            for(int i = 0; i < imgWidth; i++){
                QColor lineColor = bgColor;
                if(i >= end && i < start){
                    lineColor = QColor( (fgColor.red() < bgColor.red())?bgColor.red() - (dr*(i-end))/dc:bgColor.red() + (dr*(i-end))/dc,
                                        (fgColor.green() < bgColor.green())?bgColor.green() - (dg*(i-end))/dc:bgColor.green() + (dg*(i-end))/dc,
                                        (fgColor.blue() < bgColor.blue())?bgColor.blue() - (db*(i-end))/dc:bgColor.blue() + (db*(i-end))/dc);
                } else if(i >= start){
                    lineColor = fgColor;
                }
                QPen p = QPen(lineColor, 1);
                painter.setPen(p);
                painter.drawLine(i, 0, i, imgHeight);
            }
        }
    } else if(currentFill == Draw::fillVGradient){
        int dr = (fgColor.red() > bgColor.red())?fgColor.red() - bgColor.red():bgColor.red() - fgColor.red();
        int dg = (fgColor.green() > bgColor.green())?fgColor.green() - bgColor.green():bgColor.green() - fgColor.green();
        int db = (fgColor.blue() > bgColor.blue())?fgColor.blue() - bgColor.blue():bgColor.blue() - fgColor.blue();
        if(start <= end){
            int dc = end - start;
            for(int i = 0; i < imgHeight; i++){
                QColor lineColor = fgColor;
                if(i >= start && i < end){
                    lineColor = QColor( (fgColor.red() > bgColor.red())?fgColor.red() - (dr*(i-start))/dc:fgColor.red() + (dr*(i-start))/dc,
                                        (fgColor.green() > bgColor.green())?fgColor.green() - (dg*(i-start))/dc:fgColor.green() + (dg*(i-start))/dc,
                                        (fgColor.blue() > bgColor.blue())?fgColor.blue() - (db*(i-start))/dc:fgColor.blue() + (db*(i-start))/dc);
                } else if(i >= end){
                    lineColor = bgColor;
                }
                QPen p = QPen(lineColor, 1);
                painter.setPen(p);
                painter.drawLine(0, i, imgWidth, i);
            }
        } else {
            int dc = start - end;
            for(int i = 0; i < imgHeight; i++){
                QColor lineColor = bgColor;
                if(i >= end && i < start){
                    lineColor = QColor( (fgColor.red() < bgColor.red())?bgColor.red() - (dr*(i-end))/dc:bgColor.red() + (dr*(i-end))/dc,
                                        (fgColor.green() < bgColor.green())?bgColor.green() - (dg*(i-end))/dc:bgColor.green() + (dg*(i-end))/dc,
                                        (fgColor.blue() < bgColor.blue())?bgColor.blue() - (db*(i-end))/dc:bgColor.blue() + (db*(i-end))/dc);
                } else if(i >= start){
                    lineColor = fgColor;
                }
                QPen p = QPen(lineColor, 1);
                painter.setPen(p);
                painter.drawLine(0, i, imgWidth, i);
            }
        }
    }
}

QColor DrawView::getColorAt(int x, int y)
{
    if(x >= 0 && x < imgWidth && y >= 0 && y < imgHeight){
        QPixmap pm = QPixmap(1, 1);
        bitBlt(&pm, 0, 0, &graphic, x, y, 1, 1, CopyROP);
        QImage img = pm.convertToImage();
        return QColor(img.pixel(0, 0));
    }
    return black;
}

void DrawView::pickColor(int x, int y)
{
    QColor picked = getColorAt(x, y);
    emit colorPicked(picked);
}

void DrawView::findArea(int x, int y, QColor c, QRegion& r, int op)
{
    QRect rc = QRect(x, y, 1, 1);
    int w = graphic.height();
    int h = graphic.width();
    // expand in all directions until one point has another color or is contained in r or is outside image
    bool ex_right = true;
    bool ex_top = true;
    bool ex_left = true;
    bool ex_bottom = true;
    while(ex_right || ex_top || ex_left || ex_bottom){
        if(ex_right){
            if(rc.right() < imgWidth - 1){
                // expand right
                for(int i = 0; i < rc.height(); i++){
                    QPoint p = QPoint(rc.right() + 1, rc.top() + i);
                    if(getColorAt(p.x(), p.y()) != c || r.contains(p) || p.x() >= w){
                        ex_right = false;
                        break;
                    }
                }
                if(ex_right){
                    rc.setRight(rc.right() + 1);
                }
            } else {
                ex_right = false;
            }
        }
        if(ex_top){
            if(rc.top() > 0){
                // expand top
                for(int i = 0; i < rc.width(); i++){
                    QPoint p = QPoint(rc.left() + i, rc.top() - 1);
                    if(getColorAt(p.x(), p.y()) != c || r.contains(p) || p.y() < 0){
                        ex_top = false;
                        break;
                    }
                }
                if(ex_top){
                    rc.setTop(rc.top() - 1);
                }
            } else {
                ex_top = false;
            }
        }
        if(ex_left){
            if(rc.left() > 0){
                // expand left
                for(int i = 0; i < rc.height(); i++){
                    QPoint p = QPoint(rc.left() - 1, rc.top() + i);
                    if(getColorAt(p.x(), p.y()) != c || r.contains(p) || p.x() < 0){
                        ex_left = false;
                        break;
                    }
                }
                if(ex_left){
                    rc.setLeft(rc.left() - 1);
                }
            } else {
                ex_left = false;
            }
        }
        if(ex_bottom){
            if(rc.bottom() < imgHeight - 1){
                // expand right
                for(int i = 0; i < rc.width(); i++){
                    QPoint p = QPoint(rc.left() + i, rc.bottom() + 1);
                    if(getColorAt(p.x(), p.y()) != c || r.contains(p) || p.y() >= h){
                        ex_bottom = false;
                        break;
                    }
                }
                if(ex_bottom){
                    rc.setBottom(rc.bottom() + 1);
                }
            } else {
                ex_bottom = false;
            }
        }
    }

    QRegion add_reg = QRegion(rc.x(), rc. y(), rc.width(), rc. height());
    r += add_reg;
    // recurse into neighbourhood
    for(int i = 0; i < rc.height(); i++){
        QPoint p = QPoint(rc.right() + 1, rc.top() + i);
        if(getColorAt(p.x(), p.y()) == c && !r.contains(p) && p.x() < w){
            findArea(p.x(), p.y(), c, r, op);
        }
    }
    for(int i = 0; i < rc.width(); i++){
        QPoint p = QPoint(rc.left() + i, rc.top() - 1);
        if(getColorAt(p.x(), p.y()) == c && !r.contains(p) && p.y() > 0){
            findArea(p.x(), p.y(), c, r, op);
        }
    }
    for(int i = 0; i < rc.height(); i++){
        QPoint p = QPoint(rc.left() - 1, rc.top() + i);
        if(getColorAt(p.x(), p.y()) == c && !r.contains(p) && p.x() > 0){
            findArea(p.x(), p.y(), c, r, op);
        }
    }
    for(int i = 0; i < rc.width(); i++){
        QPoint p = QPoint(rc.left() + i, rc.bottom() + 1);
        if(getColorAt(p.x(), p.y()) == c && !r.contains(p) && p.y() < h){
            findArea(p.x(), p.y(), c, r, op);
        }
    }
}

void DrawView::setSelection(int x, int y, int w, int h)
{
    QRegion region;
    if(w != 0 && h != 0 || currentSelType == Draw::selArea){
        switch(currentSelType){
            case Draw::selRect:
            region = QRegion(x, y, w, h);
            break;
            case Draw::selCircle:
            if(w < h)
                region = QRegion(x, y + h/2 - w/2, w, w, QRegion::Ellipse);
            else
                region = QRegion(x + w/2 - h/2, y, h, h, QRegion::Ellipse);
            break;
            case Draw::selEllipse:
            region = QRegion(x, y, w, h, QRegion::Ellipse);
            break;
            case Draw::selArea:
            QApplication::setOverrideCursor(Qt::waitCursor);
            findArea(x, y, getColorAt(x, y), region);
            QApplication::restoreOverrideCursor();
            break;
        }
    }
    switch(currentSelOp){
        case 0:
        selection = region;
        break;
        case 1:
        if(!region.isEmpty())
            selection += region;
        break;
        case 2:
        if(!region.isEmpty()){
            if(selection.isEmpty())
                selection = QRegion(0, 0, imgWidth, imgHeight);
            selection -= region;
        }
        break;
    }
    emit selectionChanged();
}

void DrawView::moveSelection(int dx, int dy)
{
    int factor = zoomfactors[zoomIndex];
    QRect moveRect = QRect(trackerMoveRect.x() * 4 / factor, trackerMoveRect.y() * 4 / factor, trackerMoveRect.width() * 4 / factor, trackerMoveRect.height() * 4 / factor);
    QPixmap pm = QPixmap(moveRect.width(), moveRect.height());
    QPoint p = QPoint(0,0);
    bitBlt(&pm, p, &graphic, moveRect, CopyROP);
    QPainter painter;
    painter.begin(&graphic);
    painter.setClipRegion(selection);
    painter.fillRect(moveRect, bgColor);
    selection.translate(dx, dy);
    QRect target = selection.boundingRect();
    int tdx = 0;
    int tdy = 0;

    if(target.top() < 0 || target.left() < 0 || target.right() >= imgWidth || target.bottom() >= imgHeight){
        if(target.top() < 0){
            tdy = target.top();
            target.setTop(0);
        }
        if(target.left() < 0){
            tdx = target.left();
            target.setLeft(0);
        }
        if(target.right() >= imgWidth)
            target.setRight(imgWidth - 1);
        if(target.bottom() >= imgHeight)
            target.setBottom(imgHeight - 1);
        QRegion r = QRegion(target.left(), target.top(), target.width(), target.height());
        selection = selection.intersect(r);
        emit selectionChanged();
    }
    painter.setClipRegion(selection);

    painter.drawPixmap(target.x() + tdx, target.y() + tdy, pm);
    painter.end();
}

void DrawView::doDrawTool(int x, int y)
{
    QPainter painter;
    painter.begin(&graphic);
    painter.setPen(pen);
    brush.setStyle(NoBrush);
    painter.setBrush(brush);
    painter.setBackgroundMode(OpaqueMode);
    painter.setRasterOp(CopyROP);
    if(!selection.isEmpty())
        painter.setClipRegion(selection);

    int w = x - trackerOrigin_x;
    int h = y - trackerOrigin_y;

    TextInput* ti;
    setEdited(true);
    switch(currentTool){
        case Draw::dtCursor:
            moveSelection(w, h);
            selection_hidden = false;
            break;
        case Draw::dtSelect:
            setSelection(trackerOrigin_x, trackerOrigin_y, w, h);
            break;
        case Draw::dtLine:
            drawLine(painter, trackerOrigin_x, trackerOrigin_y, x, y);
            break;
        case Draw::dtFill:
            switch(currentFill){
                case Draw::fillSolid:
                fill(painter);
                break;
                case Draw::fillArea:
                fill(painter, x, y);
                break;
                case Draw::fillHGradient:
                fill(painter, trackerOrigin_x, x);
                break;
                case Draw::fillVGradient:
                fill(painter, trackerOrigin_y, y);
                break;
            }
            break;
        case Draw::dtPicker:
            pickColor(x, y);
            break;
        case Draw::dtText:
            ti = new TextInput(this, "Edit Text", true);
            if(ti->exec() == QDialog::Accepted){
                QString text = ti->TextEdit->text();
                painter.setFont(font);
                if(w == 0 || h == 0)
                    painter.drawText(x, y, imgWidth - x, imgHeight - y, AlignLeft|AlignTop, text);
                else
                    painter.drawText(trackerOrigin_x, trackerOrigin_y, w, h, AlignLeft|AlignTop, text);
            }
            delete ti;
            break;
        case Draw::dtPolygon:
            switch(currentShape){
                case Draw::shRect:
                    painter.drawRect(trackerOrigin_x, trackerOrigin_y, w, h);
                    break;
                case Draw::shFilledRect:
                    brush.setStyle(SolidPattern);
                    painter.setBrush(brush);
                    painter.fillRect(trackerOrigin_x, trackerOrigin_y, w, h, brush);
                    break;
                case Draw::shCircle:
                    if(w < h)
                        painter.drawArc(trackerOrigin_x, trackerOrigin_y + h/2 - w/2, w, w, 0, 5760);
                    else
                        painter.drawArc(trackerOrigin_x + w/2 - h/2, trackerOrigin_y, h, h, 0, 5760);
                    break;
                case Draw::shFilledCircle:
                    brush.setStyle(SolidPattern);
                    painter.setBrush(brush);
                    if(w < h)
                        painter.drawPie(trackerOrigin_x, trackerOrigin_y + h/2 - w/2, w, w, 0, 5760);
                    else
                        painter.drawPie(trackerOrigin_x + w/2 - h/2, trackerOrigin_y, h, h, 0, 5760);
                    break;
                case Draw::shEllipse:
                    painter.drawEllipse(trackerOrigin_x, trackerOrigin_y, w, h);
                    break;
                case Draw::shFilledEllipse:
                    brush.setStyle(SolidPattern);
                    painter.setBrush(brush);
                    painter.drawPie(trackerOrigin_x, trackerOrigin_y, w, h, 0, 5760);
                    break;
            }
            break;
    }
    painter.end();
}

void DrawView::contentsMouseMoveEvent(QMouseEvent* mev)
{
    mouse_x = mev->x();
    mouse_y = mev->y();
    if(mouse_pressed || pasting){
        track(mouse_x, mouse_y);
        ensureVisible(mouse_x, mouse_y, 8, 8);
        updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
    }
    emit mouseMoved();
}

void DrawView::contentsMousePressEvent(QMouseEvent* mev)
{
    mouse_x = mev->x();
    mouse_y = mev->y();
    mouse_pressed = true;
    startTracker(mouse_x, mouse_y);
    direction = dirNull;
    last_direction = dirNull;
    updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
    emit mousePressed();
}

void DrawView::contentsMouseReleaseEvent(QMouseEvent* mev)
{
    mouse_x = mev->x();
    mouse_y = mev->y();
    mouse_pressed = false;
    endTracker(mouse_x, mouse_y);
    updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
    emit mouseReleased();
}

/*
void DrawView::paintEvent( QPaintEvent * )
{
	QPainter paint(this);
    drawIt(&paint);
}
*/

void DrawView::setUndo()
{
    if(undoEnabled){
        bitBlt(&pmUndo, 0, 0, &graphic, 0, 0, imgWidth, imgHeight, CopyROP);
        undoRegion = QRegion(selection);
        undoState = usCanUndo;
        emit undoStateChanged(undoState);
    }
}

void DrawView::editUndo()
{
    if(undoEnabled){
        QPixmap pmTemp = QPixmap(imgWidth, imgHeight);
        bitBlt(&pmTemp, 0, 0, &graphic, 0, 0, imgWidth, imgHeight, CopyROP);
        bitBlt(&graphic, 0, 0, &pmUndo, 0, 0, imgWidth, imgHeight, CopyROP);
        bitBlt(&pmUndo, 0, 0, &pmTemp, 0, 0, imgWidth, imgHeight, CopyROP);
        QRegion rgTemp = QRegion(selection);
        selection = QRegion(undoRegion);
        undoRegion = QRegion(rgTemp);
        undoState = usCanRedo;
        image = graphic.convertToImage();
        bool g = image.isGrayscale();
        int d = image.depth();
        if(d != depth || g != gray){
            depth = d;
            gray = g;
            emit colorStateChanged(depth, gray);
        }
        if(isScaled)
            scale();
        updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
        emit undoStateChanged(undoState);
    }
}

void DrawView::editRedo()
{
    if(undoEnabled){
        QPixmap pmTemp = QPixmap(imgWidth, imgHeight);
        bitBlt(&pmTemp, 0, 0, &graphic, 0, 0, imgWidth, imgHeight, CopyROP);
        bitBlt(&graphic, 0, 0, &pmUndo, 0, 0, imgWidth, imgHeight, CopyROP);
        bitBlt(&pmUndo, 0, 0, &pmTemp, 0, 0, imgWidth, imgHeight, CopyROP);
        QRegion rgTemp = QRegion(selection);
        selection = QRegion(undoRegion);
        undoRegion = QRegion(rgTemp);
        undoState = usCanUndo;
        image = graphic.convertToImage();
        bool g = image.isGrayscale();
        int d = image.depth();
        if(d != depth || g != gray){
            depth = d;
            gray = g;
            emit colorStateChanged(depth, gray);
        }
        if(isScaled)
            scale();
        updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
        emit undoStateChanged(undoState);
    }
}

void DrawView::editCut()
{
    QRect src = selection.boundingRect();
    clipBits.resize(src.width(), src.height());
    QPoint p = QPoint(0,0);
    bitBlt(&clipBits, p, &graphic, src, CopyROP);
    QPainter painter;
    painter.begin(&graphic);
    painter.setClipRegion(selection);
    painter.fillRect(src, bgColor);
    clipRegion = selection;
    clipRegion.translate(-1 * src.x(), -1 * src.y());
    clearMask();
    emit clipStateChanged();
}

void DrawView::editCopy()
{
    QRect src = selection.boundingRect();
    clipBits.resize(src.width(), src.height());
    QPoint p = QPoint(0,0);
    bitBlt(&clipBits, p, &graphic, src, CopyROP);
    clipRegion = selection;
    clipRegion.translate(-1 * src.x(), -1 * src.y());
    emit clipStateChanged();
}

void DrawView::paste(int x, int y)
{
    int factor = zoomfactors[zoomIndex];
    QRect moveRect = QRect(trackerMoveRect.x() * 4 / factor, trackerMoveRect.y() * 4 / factor, trackerMoveRect.width() * 4 / factor, trackerMoveRect.height() * 4 / factor);
    QRegion targetRegion = clipRegion;
    targetRegion.translate(x, y);
    QPainter painter;
    painter.begin(&graphic);
    painter.setClipRegion(targetRegion);
    painter.drawPixmap(x, y, clipBits);
    painter.end();
    pasting = false;
    setEdited(true);
    if(isScaled)
        scale();
    updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
}

void DrawView::editPaste()
{
    clearMask();
    trackerOrigin_x = 0;
    trackerOrigin_y = 0;
    last_x = 0;
    last_y = 0;
    int factor = zoomfactors[zoomIndex];
    trackerMoveRect = clipRegion.boundingRect();
    if(isScaled){
        trackerMoveRect.setLeft(trackerMoveRect.left() * factor / 4);
        trackerMoveRect.setTop(trackerMoveRect.top() * factor / 4);
        trackerMoveRect.setRight(trackerMoveRect.right() * factor / 4);
        trackerMoveRect.setBottom(trackerMoveRect.bottom() * factor / 4);
    }
    drawTracker(0, 0, trMove, false);
    pasting = true;
}

void DrawView::hideMask(bool h)
{
    if(scToggle)
        timeoutUpdate();
    selection_user_hidden = h;
}

void DrawView::invertMask()
{
    setUndo();
    QRegion new_sel = QRegion(0,0,imgWidth,imgHeight);
    new_sel -= selection;
    selection = new_sel;
    emit selectionChanged();
}

void DrawView::clearMask()
{
    if(scToggle)
        timeoutUpdate();
    setUndo();
    selection = QRegion();
    emit selectionChanged();
}

void DrawView::zoomIn()
{
    if(zoomIndex < (nzoomfactors - 1)){
        zoomIndex++;
        scale();
        updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
    }
    if(zoomIndex >= (nzoomfactors - 1))
        emit zoomStateChanged(zsZoomOutOnly);
    else
        emit zoomStateChanged(zsCanZoom);
}

void DrawView::zoomOut()
{
    if(zoomIndex > 0){
        zoomIndex--;
        scale();
        updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
    }
    if(zoomIndex < 1)
        emit zoomStateChanged(zsZoomInOnly);
    else
        emit zoomStateChanged(zsCanZoom);
}

void DrawView::zoom(int level)
{
    if(level >= 0 && level < nzoomfactors){
        zoomIndex = level;
        scale();
        updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
    }
    if(zoomIndex < 1)
        emit zoomStateChanged(zsZoomInOnly);
    else if(zoomIndex >= (nzoomfactors - 1))
        emit zoomStateChanged(zsZoomOutOnly);
    else
        emit zoomStateChanged(zsCanZoom);
}


void DrawView::scale()
{
    int factor = zoomfactors[zoomIndex];
    if(scToggle)
        timeoutUpdate();
    selection_hidden = true;
    if(factor == 4){
        isScaled = false;
        pmScaled.resize(0,0);
        scaledWidth = imgWidth;
        scaledHeight = imgHeight;
    } else {
        image = graphic.convertToImage();
        scaledWidth = (imgWidth * factor)/4;
        scaledHeight = (imgHeight * factor)/4;
        pmScaled.convertFromImage( image.smoothScale( scaledWidth, scaledHeight ) );
        isScaled = true;
    }
    resizeContents(scaledWidth, scaledHeight);
    selection_hidden = false;
}

void DrawView::resizeImage()
{
    NewPic* np = new NewPic(this, "resize picture", true);
    np->sbWidth->setValue(imgWidth);
    np->sbHeight->setValue(imgHeight);
    int result = np->exec();
    if(result == QDialog::Accepted){
        int w = np->sbWidth->value();
        int h = np->sbHeight->value();
        image = graphic.convertToImage();
        init(image.smoothScale(w, h));
        setEdited(true);
    }
    delete np;
}

void DrawView::hFlip()
{
    image = graphic.convertToImage();
    setImage(image.mirror(TRUE,FALSE));
}

void DrawView::vFlip()
{
    image = graphic.convertToImage();
    setImage(image.mirror(FALSE,TRUE));
}

void DrawView::rot180()
{
    image = graphic.convertToImage();
    setImage(image.mirror(TRUE,TRUE));
}

void DrawView::rot90()
{
    QImage oldimage, newimage;
    uchar *oldbits, *newbits;
    int i, j, p;
    int w, h;

    image = graphic.convertToImage();
    oldimage = image.convertDepth(32);
    w = oldimage.height();
    h = oldimage.width();
    newimage = QImage( w, h, 32);

    oldbits = oldimage.bits();
    newbits = newimage.bits();

    for (i=0; i < w ; i++)
        for (j=0; j < h; j++)
        for (p = 0 ; p < 4 ; p++)
            newbits[(j * w + i) * 4 + p] = oldbits[ ((i + 1) * h  - j ) * 4 + p];

    setImage(newimage);
    scale();
}

void DrawView::darken()
{
    image = graphic.convertToImage();
    QImage newimage = image.convertDepth(32);
    int h = newimage.height();
    int w = newimage.width();
    QPoint p;
    for(int i = 0; i < w ; i++){
        for(int j = 0; j < h; j++){
            p.setX(i);
            p.setY(j);
            if(selection.isEmpty() || selection.contains(p)){
                QColor c = QColor(newimage.pixel(i, j));
                newimage.setPixel(i, j, c.dark(110).rgb());
            }
        }
    }
    setImage(newimage);
}

void DrawView::lighten()
{
    image = graphic.convertToImage();
    QImage newimage = image.convertDepth(32);
    int h = newimage.height();
    int w = newimage.width();
    QPoint p;
    for(int i = 0; i < w ; i++){
        for(int j = 0; j < h; j++){
            p.setX(i);
            p.setY(j);
            if(selection.isEmpty() || selection.contains(p)){
                QColor c = QColor(newimage.pixel(i, j));
                newimage.setPixel(i, j, c.light(110).rgb());
            }
        }
    }
    setImage(newimage);
}

void DrawView::sharpen()
{
    int f_matrix[3][3];
    f_matrix[0][0] = -1;
    f_matrix[0][1] = -1;
    f_matrix[0][2] = -1;

    f_matrix[1][0] = -1;
    f_matrix[1][1] = 12;
    f_matrix[1][2] = -1;

    f_matrix[2][0] = -1;
    f_matrix[2][1] = -1;
    f_matrix[2][2] = -1;
    filter(f_matrix);
}

void DrawView::blur()
{
    int f_matrix[3][3];
    f_matrix[0][0] = 1;
    f_matrix[0][1] = 4;
    f_matrix[0][2] = 1;

    f_matrix[1][0] = 4;
    f_matrix[1][1] = 16;
    f_matrix[1][2] = 4;

    f_matrix[2][0] = 1;
    f_matrix[2][1] = 4;
    f_matrix[2][2] = 1;
    filter(f_matrix);
}

void DrawView::emboss()
{
    int f_matrix[3][3];
    f_matrix[0][0] =  2;
    f_matrix[0][1] =  2;
    f_matrix[0][2] =  0;

    f_matrix[1][0] =  2;
    f_matrix[1][1] =  0;
    f_matrix[1][2] = -2;

    f_matrix[2][0] =  0;
    f_matrix[2][1] = -2;
    f_matrix[2][2] = -2;
    filter(f_matrix, 8, 120);
}

void DrawView::filter(int f_matrix[3][3], int fact, int bias)
{
    if(depth == 1)return;
    if(depth == 8){
        if(!gray) return;
    }
    image = graphic.convertToImage();
    QImage newimage = image.convertDepth(32);
    int he = newimage.height();
    int wi = newimage.width();
    filterFu(&newimage, selection, f_matrix, fact, bias);
    if(selection.isEmpty()){
        /* borders and corners: */
        for(int i = 1; i < (wi - 1) ; i++){
            QColor c = QColor(newimage.pixel(i, 1));
            newimage.setPixel(i, 0, c.rgb());
            c = QColor(newimage.pixel(i, he - 2));
            newimage.setPixel(i, he - 1, c.rgb());
        }
        for(int j = 1; j < (he - 1); j++){
            QColor c = QColor(newimage.pixel(1, j));
            newimage.setPixel(0, j, c.rgb());
            c = QColor(newimage.pixel(wi - 2, j));
            newimage.setPixel(wi - 1, j, c.rgb());
        }
        QColor c = QColor(newimage.pixel(1, 1));
        newimage.setPixel(0, 0, c.rgb());
        c = QColor(newimage.pixel(wi - 2, 1));
        newimage.setPixel(wi - 1, 0, c.rgb());
        c = QColor(newimage.pixel(wi - 2, he - 2));
        newimage.setPixel(wi - 1, he - 1, c.rgb());
        c = QColor(newimage.pixel(1, he - 2));
        newimage.setPixel(0, he - 1, c.rgb());
    }
    if(gray){
        QImage grayimage = newimage.convertDepth(8);
        setImage(grayimage);
    } else {
        setImage(newimage);
    }
}

void DrawView::filterFu(QImage *newimage, QRegion sel, int f_matrix[3][3], int fact, int bias)
{
    int he = newimage->height();
    int wi = newimage->width();
    int h;
    int s;
    int v;
    int v_arr[3][3];
    QPoint p;
    int factor = fact;
    if(fact == 0)
        factor = f_matrix[0][0] + f_matrix[0][1] + f_matrix[0][2] + f_matrix[1][0] + f_matrix[1][1] + f_matrix[1][2] + f_matrix[2][0] + f_matrix[2][1] + f_matrix[2][2];
    for(int i = 1; i < (wi - 1) ; i++){
        for(int j = 1; j < (he - 1); j++){
            p.setX(i);
            p.setY(j);
            if(sel.isEmpty() || sel.contains(p)){
                for(int k = 0; k < 3; k++){
                    for(int l = 0; l < 3; l++){
                        QColor mxc = QColor(newimage->pixel(i - 1 + k, j - 1 + l));
                        if(gray){
                            v_arr[k][l] = (mxc.red()*11 + mxc.green()*16 + mxc.blue()*5)/32;
                        } else {
                            mxc.hsv(&h, &s, &v);
                            v_arr[k][l] = v;
                        }
                    }
                }
                QColor c = QColor(newimage->pixel(i, j));
                if(!gray)c.hsv(&h, &s, &v);
                v = ((f_matrix[0][0] * v_arr[0][0]) + (f_matrix[0][1] * v_arr[0][1]) + (f_matrix[0][2] * v_arr[0][2]) + (f_matrix[1][0] * v_arr[1][0]) + (f_matrix[1][1] * v_arr[1][1]) + (f_matrix[1][2] * v_arr[1][2]) + (f_matrix[2][0] * v_arr[2][0]) + (f_matrix[2][1] * v_arr[2][1]) + (f_matrix[2][2] * v_arr[2][2]))/factor + bias;
                if(v < 0)v = 0;
                if(v > 255)v = 255;
                if(gray){
                    c.setRgb(v, v, v);
                } else {
                    c.setHsv(h, s, v);
                }
                newimage->setPixel(i, j, c.rgb());
            }
        }
    }
}

void DrawView::convertToGray()
{
    image = graphic.convertToImage();
    QImage newimage = image.convertDepth(32);
    int h = newimage.height();
    int w = newimage.width();
    for(int i = 0; i < w ; i++){
        for(int j = 0; j < h; j++){
            QColor c = QColor(image.pixel(i, j));
            int gray = (c.red()*11 + c.green()*16 + c.blue()*5)/32;
            c.setRgb(gray, gray, gray);
            newimage.setPixel(i, j, c.rgb());
        }
    }
    QImage grayimage = newimage.convertDepth(8);
    gray = true;
    depth = 8;
    setImage(grayimage);
    emit colorStateChanged(depth, gray);
}

void DrawView::convertToMono()
{
    image = graphic.convertToImage();
    QImage newimage = image.convertDepth(1);
    gray = false;
    depth = 1;
    setImage(newimage);
    emit colorStateChanged(depth, gray);
}

void DrawView::convertToIndexed()
{
    image = graphic.convertToImage();
    QImage newimage = image.convertDepth(8);
    gray = false;
    depth = 8;
    setImage(newimage);
    emit colorStateChanged(depth, gray);
}

void DrawView::convertToRGB()
{
    image = graphic.convertToImage();
    QImage newimage = image.convertDepth(32);
    gray = false;
    depth = 32;
    setImage(newimage);
    emit colorStateChanged(depth, gray);
}

void DrawView::setImage(QImage img)
{
    image = img;
    int h = image.height();
    int w = image.width();
    if(w == imgWidth && h == imgHeight)
        setUndo();
    graphic.convertFromImage(image);
    if(w != imgWidth || h != imgHeight){
        clearMask();
        imgWidth = w;
        imgHeight = h;
        undoState = usCantDo;
        resizeContents(w, h);
        emit undoStateChanged(undoState);
    }
    setEdited(true);
    if(isScaled)
        scale();
    updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
}

void DrawView::drawSelection(QPainter *p)
{
    QArray<QRect> regionrects = selection.rects();
    QPen oldPen = p->pen();
    RasterOp oldROP = p->rasterOp();
    QPen sPen = QPen(scToggle?white:black, 1);
    p->setPen(sPen);
    p->setRasterOp(XorROP);
    int factor = zoomfactors[zoomIndex];
    for(int i = 0; i < regionrects.size(); i++)
        p->drawRect(regionrects[i].x() * factor / 4,regionrects[i].y() * factor / 4,regionrects[i].width() * factor / 4,regionrects[i].height() * factor / 4+1);
    p->setPen(oldPen);
    p->setRasterOp(oldROP);
}

void DrawView::timeoutUpdate()
{
    if(!selection.isEmpty() && !selection_hidden && !selection_user_hidden){
        if(scToggle)scToggle = false;
        else scToggle = true;
        updateContents(contentsX(), contentsY(), visibleWidth(), visibleHeight());
    }
}


void DrawView::drawContents(QPainter* p, int clipx, int clipy, int clipw, int cliph)
{
    QPixmap pm = QPixmap((clipw < scaledWidth)?clipw:scaledWidth, (cliph < scaledHeight)?cliph:scaledHeight);
    if(isScaled)
        bitBlt(&pm, 0, 0, &pmScaled, clipx, clipy, (clipw < scaledWidth)?clipw:scaledWidth, (cliph < scaledHeight)?cliph:scaledHeight, CopyROP);
    else
        bitBlt(&pm, 0, 0, &graphic, clipx, clipy, (clipw < imgWidth)?clipw:imgWidth, (cliph < imgHeight)?cliph:imgHeight, CopyROP);
    p->drawPixmap(clipx,clipy,pm);
    if(clipw > scaledWidth){
        p->fillRect(clipx + scaledWidth, clipy, clipw - scaledWidth, cliph, QColor( 85, 85, 127));
    }
    if(cliph > scaledHeight){
        p->fillRect(clipx, clipy + scaledHeight, clipw, cliph - scaledHeight, QColor( 85, 85, 127));
    }
    if(!selection.isEmpty() && !selection_hidden && !selection_user_hidden){
        drawSelection(p);
    }
}
