/**********************************************************************
** Copyright (C) 2002 Edgar Hindemith.  All rights reserved.
**
** email: ed@dom.de
**
** This file is part of the qpPhoto picture editor.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/
#include "draw.h"
#include "drawview.h"
#include "saveAsDlg.h"
#include "AboutDialog.h"
#include "qpPhoto.h"

#include <qpe/global.h>
#include <qpe/config.h>
#include <qpe/fileselector.h>
#include <qpe/applnk.h>
#include <qpe/resource.h>
//#include <qpe/config.h>
#include <qpe/qpeapplication.h>
#include <qpe/qpemenubar.h>
#include <qpe/qpetoolbar.h>
//#include <qpe/finddialog.h>

#include <qaction.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qobjectlist.h>
#include <qpopupmenu.h>
#include <qtoolbutton.h>
#include <qwidgetstack.h>

#include <stdlib.h> //getenv

static int u_id = 1;
static int get_unique_id()
{
    return u_id++;
}

qpPhoto::qpPhoto( QWidget *parent, const char *name, WFlags f )
    : QMainWindow( parent, name, f ), bFromDocView( FALSE )
{
    doc = 0;

    setToolBarsMovable( FALSE );

    setIcon( Resource::loadPixmap( "qpPhoto" ) );

    QPEToolBar *bar = new QPEToolBar( this );
    bar->setHorizontalStretchable( TRUE );
    menu = bar;

    stack = new QWidgetStack( this );
    setCentralWidget( stack );
    draw = new Draw( stack );

    QPEMenuBar *mb = new QPEMenuBar( bar );
    QPopupMenu *mnFile = new QPopupMenu( this );
    QPopupMenu *mnEdit = new QPopupMenu( this );
    QPopupMenu *mnView = new QPopupMenu( this );
    QPopupMenu *mnImage = new QPopupMenu( this );
    QPopupMenu *mnConvert = new QPopupMenu( this );
    QPopupMenu *mnFilter = new QPopupMenu( this );
    QPopupMenu *mnHelp = new QPopupMenu( this );

    mnZoom = new QPopupMenu( this );

    bar = new QPEToolBar( this );
    editBar = bar;

    QAction *a = new QAction( tr( "New" ), Resource::loadPixmap( "new" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fileNew() ) );
    a->addTo( mnFile );

    a = new QAction( tr( "Open" ), Resource::loadPixmap( "fileopen" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
    a->addTo( mnFile );

    mnFile->insertSeparator();

    /*a = new QAction( tr( "Save" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fileSave() ) );
    a->addTo( mnFile );*/

    a = new QAction( tr( "Save as ..." ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fileSaveAs() ) );
    a->addTo( mnFile );

    mnFile->insertSeparator();

    a = new QAction( tr( "Info ..." ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( info() ) );
    a->addTo( mnFile );

    ua = new QAction( tr("Undo"), QString::null, 0, this, 0 );
    connect( ua, SIGNAL( activated() ), draw->dv, SLOT( editUndo() ) );
    ua->addTo( mnEdit );

    ra = new QAction( tr("Redo"), QString::null, 0, this, 0 );
    connect( ra, SIGNAL( activated() ), draw->dv, SLOT( editRedo() ) );
    ra->addTo( mnEdit );

    eua = new QAction( tr("Enable undo"), QString::null, 0, this, 0 );
    connect( eua, SIGNAL( toggled(bool) ), this, SLOT( enableUndo(bool) ) );
    eua->setToggleAction(TRUE);
    eua->addTo( mnEdit );

    mnEdit->insertSeparator();

    cta = new QAction( tr( "Cut" ), Resource::loadPixmap( "cut" ), QString::null, 0, this, 0 );
    connect( cta, SIGNAL( activated() ), draw->dv, SLOT( editCut() ) );
    cta->addTo( editBar );
    cta->addTo( mnEdit );

    cpa = new QAction( tr( "Copy" ), Resource::loadPixmap( "copy" ), QString::null, 0, this, 0 );
    connect( cpa, SIGNAL( activated() ), draw->dv, SLOT( editCopy() ) );
    cpa->addTo( editBar );
    cpa->addTo( mnEdit );

    pa = new QAction( tr( "Paste" ), Resource::loadPixmap( "paste" ), QString::null, 0, this, 0 );
    connect( pa, SIGNAL( activated() ), draw->dv, SLOT( editPaste() ) );
    pa->addTo( editBar );
    pa->addTo( mnEdit );

    mnEdit->insertSeparator();

    hma = new QAction( tr("Hide mask"), QString::null, 0, this, 0 );
    connect( hma, SIGNAL( toggled(bool) ), draw->dv, SLOT( hideMask(bool) ) );
    hma->setToggleAction(TRUE);
    hma->addTo( mnEdit );

    ima = new QAction( tr("Invert mask"), QString::null, 0, this, 0 );
    connect( ima, SIGNAL( activated() ), draw->dv, SLOT( invertMask() ) );
    ima->addTo( mnEdit );

    cma = new QAction( tr("Clear mask"), QString::null, 0, this, 0 );
    connect( cma, SIGNAL( activated() ), draw->dv, SLOT( clearMask() ) );
    cma->addTo( mnEdit );

    zia = new QAction( tr("Zoom in"), QString::null, 0, this, 0 );
    connect( zia, SIGNAL( activated() ), draw->dv, SLOT( zoomIn() ) );
    zia->addTo( mnView );

    zoa = new QAction( tr("Zoom out"), QString::null, 0, this, 0 );
    connect( zoa, SIGNAL( activated() ), draw->dv, SLOT( zoomOut() ) );
    zoa->addTo( mnView );

    mnView->insertSeparator();

    mnView->insertItem( tr( "Zoom level" ), mnZoom);
    mnZoom->setCheckable(true);
    z25 = mnZoom->insertItem( tr("25%"), this, SLOT( zoom25() ) );
    z50 = mnZoom->insertItem( tr("50%"), this, SLOT( zoom50() ) );
    z75 = mnZoom->insertItem( tr("75%"), this, SLOT( zoom75() ) );
    z100 = mnZoom->insertItem( tr("100%"), this, SLOT( zoom100() ) );
    z200 = mnZoom->insertItem( tr("200%"), this, SLOT( zoom200() ) );
    z400 = mnZoom->insertItem( tr("400%"), this, SLOT( zoom400() ) );
    z800 = mnZoom->insertItem( tr("800%"), this, SLOT( zoom800() ) );

    mnImage->insertItem( tr( "Convert To" ), mnConvert);

    cvm = new QAction( tr( "Monochrome" ), QString::null, 0, this, 0 );
    connect( cvm, SIGNAL( activated() ), draw->dv, SLOT( convertToMono() ) );
    cvm->addTo( mnConvert );

    cvg = new QAction( tr( "Grayscale" ), QString::null, 0, this, 0 );
    connect( cvg, SIGNAL( activated() ), draw->dv, SLOT( convertToGray() ) );
    cvg->addTo( mnConvert );

    cvi = new QAction( tr( "Indexed" ), QString::null, 0, this, 0 );
    connect( cvi, SIGNAL( activated() ), draw->dv, SLOT( convertToIndexed() ) );
    cvi->addTo( mnConvert );

    cvr = new QAction( tr( "RGB" ), QString::null, 0, this, 0 );
    connect( cvr, SIGNAL( activated() ), draw->dv, SLOT( convertToRGB() ) );
    cvr->addTo( mnConvert );

    a = new QAction( tr( "Resize" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), draw->dv, SLOT( resizeImage() ) );
    a->addTo( mnImage );

    a = new QAction( tr( "Horizontal flip" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), draw->dv, SLOT( hFlip() ) );
    a->addTo( mnImage );

    a = new QAction( tr( "Vertical flip"), QString::null, 0, this, 0);
    connect( a, SIGNAL( activated() ), draw->dv, SLOT( vFlip() ) );
    a->addTo( mnImage );

    a = new QAction( tr( "Rotate 180" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), draw->dv, SLOT( rot180() ) );
    a->addTo( mnImage );

    a = new QAction( tr( "Rotate 90"), QString::null, 0, this, 0);
    connect( a, SIGNAL( activated() ), draw->dv, SLOT( rot90() ) );
    a->addTo( mnImage );

    a = new QAction( tr( "Lighten" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), draw->dv, SLOT( lighten() ) );
    a->addTo( mnImage );

    a = new QAction( tr( "Darken"), QString::null, 0, this, 0);
    connect( a, SIGNAL( activated() ), draw->dv, SLOT( darken() ) );
    a->addTo( mnImage );

    mnImage->insertItem( tr( "Filter" ), mnFilter);

    fbl = new QAction( tr( "Blur"), QString::null, 0, this, 0);
    connect( fbl, SIGNAL( activated() ), draw->dv, SLOT( blur() ) );
    fbl->addTo( mnFilter );

    fsh = new QAction( tr( "Sharpen"), QString::null, 0, this, 0);
    connect( fsh, SIGNAL( activated() ), draw->dv, SLOT( sharpen() ) );
    fsh->addTo( mnFilter );

    fem = new QAction( tr( "Emboss"), QString::null, 0, this, 0);
    connect( fem, SIGNAL( activated() ), draw->dv, SLOT( emboss() ) );
    fem->addTo( mnFilter );

    a = new QAction( tr( "About ..." ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( about() ) );
    a->addTo( mnHelp );


    mb->insertItem( tr( "File" ), mnFile );
    mb->insertItem( tr( "Edit" ), mnEdit );
    mb->insertItem( tr( "View" ), mnView );
    mb->insertItem( tr( "Image" ), mnImage );
    mb->insertItem( tr( "Help" ), mnHelp );


    fileSelector = new FileSelector( "image/*", stack, "fileselector" ,
	    TRUE, FALSE );
    connect( fileSelector, SIGNAL( closeMe() ), this, SLOT( closeFileSelector() ) );
    connect( fileSelector, SIGNAL( newSelected( const DocLnk &) ), this, SLOT( newFile( const DocLnk & ) ) );
    connect( fileSelector, SIGNAL( fileSelected( const DocLnk &) ), this, SLOT( openFile( const DocLnk & ) ) );
    fileOpen();

    connect( draw->dv, SIGNAL ( zoomStateChanged(int) ), this, SLOT( zoomStateChanged(int) ) );
    connect( draw->dv, SIGNAL ( undoStateChanged(int) ), this, SLOT( undoStateChanged(int) ) );
    connect( draw->dv, SIGNAL ( selectionChanged() ), this, SLOT( maskStateChanged() ) );
    connect( draw->dv, SIGNAL ( clipStateChanged() ), this, SLOT( clipStateChanged() ) );
    connect( draw->dv, SIGNAL ( colorStateChanged(int, bool) ), this, SLOT( colorStateChanged(int, bool) ) );

    stack->addWidget( draw, get_unique_id() );

    bool undoEnabled = readUndoConfig();
    draw->dv->enableUndo(undoEnabled);
    eua->setOn(undoEnabled);

    //resize( 200, 300 );
}

qpPhoto::~qpPhoto()
{
    save();
}

void qpPhoto::about()
{
    AboutDialog* ad = new AboutDialog(this, "About qpPhoto", true);
    ad->exec();
}

void qpPhoto::info()
{
    QString name = "";
    QString file = "";
    QString type = "";
    QString depth = "";
    QString gray = "";
    QString imgInfo = "";

    name.sprintf("Name:  %s\n", (doc->name().isEmpty())?"Unnamed":(const char*)doc->name());
    if(!doc->name().isEmpty()){
        QFileInfo fi(doc->file());
        if(fi.exists()){
            int size = fi.size();
            QString ext = "";
            ext.sprintf("%s%s",(fi.extension().isEmpty())?"":".", (fi.extension().isEmpty())?"":(const char*)fi.extension());
            if(size < 1024)
                file.sprintf("File:  %s%s %d B\n", (const char*)fi.baseName(), (const char*)ext, size);
            else
                file.sprintf("File:  %s%s %.1f KB\n", (const char*)fi.baseName(), (const char*)ext, (double)size/1024.0);
        }
    }
    type.sprintf("\nType:  %s\n", (const char*)doc->type());
    if(draw->dv->isGray()){
        gray = " grayscale";
    }
    depth.sprintf("Depth: %d bpp%s\n", draw->dv->getDepth(), (const char*)gray);
    imgInfo = name + file + type + depth;
    QMessageBox::about( this, tr( "Image-Info" ), imgInfo);
}

bool qpPhoto::readUndoConfig()
{
    Config *cf = new Config("qpPhoto");
    bool ret = false;
    if(cf->isValid()){
        cf->setGroup("edit");
        if(cf->hasKey("undo"))
            ret = cf->readBoolEntry("undo");
        else
            cf->writeEntry("undo", 0);
    }
    delete cf;
    return ret;
}

void qpPhoto::writeUndoConfig(bool enable)
{
    Config *cf = new Config("qpPhoto");
    cf->setGroup("edit");
    cf->writeEntry("undo", (int)enable);
    delete cf;
}

void qpPhoto::enableUndo(bool enable)
{
    writeUndoConfig(enable);
    draw->dv->enableUndo(enable);
}

void qpPhoto::closeEvent( QCloseEvent *e )
{
    if ( stack->visibleWidget() == draw && !bFromDocView ) {
        e->ignore();
        fileRevert();
    } else {
        bFromDocView = FALSE;
        e->accept();
    }
}

void qpPhoto::zoom25()
{
    draw->dv->zoom(0);
}

void qpPhoto::zoom50()
{
    draw->dv->zoom(1);
}

void qpPhoto::zoom75()
{
    draw->dv->zoom(2);
}

void qpPhoto::zoom100()
{
    draw->dv->zoom(3);
}

void qpPhoto::zoom200()
{
    draw->dv->zoom(4);
}

void qpPhoto::zoom400()
{
    draw->dv->zoom(5);
}

void qpPhoto::zoom800()
{
    draw->dv->zoom(6);
}

void qpPhoto::zoomStateChanged(int zs)
{
    zia->setEnabled(zs == DrawView::zsCanZoom || zs == DrawView::zsZoomInOnly);
    zoa->setEnabled(zs == DrawView::zsCanZoom || zs == DrawView::zsZoomOutOnly);
    int level = draw->dv->getZoomIndex();
    if(mnZoom->isItemChecked(z25)){
        if(level != 0)
            mnZoom->setItemChecked(z25, false);
    } else {
        if(level == 0)
            mnZoom->setItemChecked(z25, true);
    }
    if(mnZoom->isItemChecked(z50)){
        if(level != 1)
            mnZoom->setItemChecked(z50, false);
    } else {
        if(level == 1)
            mnZoom->setItemChecked(z50, true);
    }
    if(mnZoom->isItemChecked(z75)){
        if(level != 2)
            mnZoom->setItemChecked(z75, false);
    } else {
        if(level == 2)
            mnZoom->setItemChecked(z75, true);
    }
    if(mnZoom->isItemChecked(z100)){
        if(level != 3)
            mnZoom->setItemChecked(z100, false);
    } else {
        if(level == 3)
            mnZoom->setItemChecked(z100, true);
    }
    if(mnZoom->isItemChecked(z200)){
        if(level != 4)
            mnZoom->setItemChecked(z200, false);
    } else {
        if(level == 4)
            mnZoom->setItemChecked(z200, true);
    }
    if(mnZoom->isItemChecked(z400)){
        if(level != 5)
            mnZoom->setItemChecked(z400, false);
    } else {
        if(level == 5)
            mnZoom->setItemChecked(z400, true);
    }
    if(mnZoom->isItemChecked(z800)){
        if(level != 6)
            mnZoom->setItemChecked(z800, false);
    } else {
        if(level == 6)
            mnZoom->setItemChecked(z800, true);
    }
    updateCaption();
}

void qpPhoto::undoStateChanged(int us)
{
    ua->setEnabled(us == DrawView::usCanUndo);
    ra->setEnabled(us == DrawView::usCanRedo);
}

void qpPhoto::maskStateChanged()
{
    hma->setEnabled(!draw->dv->isSelectionEmpty());
    ima->setEnabled(!draw->dv->isSelectionEmpty());
    cma->setEnabled(!draw->dv->isSelectionEmpty());
    cpa->setEnabled(!draw->dv->isSelectionEmpty());
    cta->setEnabled(!draw->dv->isSelectionEmpty());
}

void qpPhoto::clipStateChanged()
{
    pa->setEnabled(!draw->dv->isClipEmpty());
}

void qpPhoto::colorStateChanged(int depth, bool gray)
{
    switch(depth){
        case 1:
        cvm->setEnabled(false);
        cvg->setEnabled(true);
        cvi->setEnabled(true);
        cvr->setEnabled(true);
        fbl->setEnabled(false);
        fsh->setEnabled(false);
        fem->setEnabled(false);
        break;
        case 8:
        if(gray){
            cvm->setEnabled(true);
            cvg->setEnabled(false);
            cvi->setEnabled(true);
            cvr->setEnabled(true);
            fbl->setEnabled(true);
            fsh->setEnabled(true);
            fem->setEnabled(true);
        } else {
            cvm->setEnabled(true);
            cvg->setEnabled(true);
            cvi->setEnabled(false);
            cvr->setEnabled(true);
            fbl->setEnabled(false);
            fsh->setEnabled(false);
            fem->setEnabled(false);
        }
        break;
        case 32:
        cvm->setEnabled(true);
        cvg->setEnabled(true);
        cvi->setEnabled(true);
        cvr->setEnabled(false);
        fbl->setEnabled(true);
        fsh->setEnabled(true);
        fem->setEnabled(true);
        break;
        default:
        cvm->setEnabled(false);
        cvg->setEnabled(false);
        cvi->setEnabled(false);
        cvr->setEnabled(false);
        fbl->setEnabled(false);
        fsh->setEnabled(false);
        fem->setEnabled(false);
        break;
    }
}

void qpPhoto::setDocument(const QString& fileref)
{
    bFromDocView = TRUE;
    openFile(DocLnk(fileref));
    closeFileSelector();
}

void qpPhoto::fileNew()
{
    save();
    newFile(DocLnk());
}

void qpPhoto::fileRevert()
{
    clear();
    fileOpen();
}

void qpPhoto::fileOpen()
{
    if ( !save() ) {
	    if ( QMessageBox::critical( this, tr( "Unable to save" ),
				    tr( "qpPhoto was unable to\n"
					"save your changes.\n"
                    "Unsupported format or out of space.\n"
					"Free some space or choose another\n"
                    "image format and try again.\n"
					"\nContinue anyway?" ),
				    QMessageBox::Yes|QMessageBox::Escape,
				    QMessageBox::No|QMessageBox::Default ) != QMessageBox::Yes )
	        return;
	    else {
	        delete doc;
	        doc = 0;
	    }
    }
    menu->hide();
    editBar->hide();
    clearWState (WState_Reserved1 );
    stack->raiseWidget( fileSelector );
    fileSelector->reread();
    updateCaption();
}

void qpPhoto::accept()
{
    fileOpen();
}

void qpPhoto::newFile( const DocLnk &f )
{
    DocLnk nf = f;
    nf.setType("image/png");
    clear(true);
    stack->raiseWidget( draw );
    setWState (WState_Reserved1 );
    draw->init();
    draw->setFocus();
    doc = new DocLnk(nf);
    updateCaption();
}

void qpPhoto::openFile( const QString &f )
{
    bFromDocView = TRUE;
    DocLnk nf;
    const char * format = QImageIO::imageFormat(f);
    QString fStr;
    fStr.sprintf("image/%s", (format)?format:"unsupported");
    if(fStr.lower() == "image/xpm"){
        fStr = "image/x-xpixmap";
    }
    nf.setType(fStr.lower());
    nf.setFile(f);
    openFile(nf);
    closeFileSelector();
    // Show filename in caption
    updateCaption( f );
}

void qpPhoto::openFile( const DocLnk &f )
{
    save();
    clear();
    stack->raiseWidget( draw );
    setWState (WState_Reserved1 );
    draw->setFocus();
    if ( doc )
        delete doc;
    doc = new DocLnk(f);
    draw->init();
    draw->dv->init(f.file());
    updateCaption();
}

void qpPhoto::closeFileSelector()
{
    if ( !doc )
        close();
    fileSelector->hide();
    menu->show();
    editBar->show();
    updateCaption();
}

void qpPhoto::fileSave()
{
    save();
}

void qpPhoto::fileSaveAs()
{
    save(true);
}

bool qpPhoto::save(bool saveAs)
{
    // case of nothing to save...
    if ( !doc ){
        return true;
    }

    if ( !draw->dv->edited() && !saveAs) {
        delete doc;
        doc = 0;
        return true;
    }
    QString ext = "";
    const char* format = draw->dv->getFormat();
    int quality = -1;
    if ( doc->name().isEmpty() || saveAs ) {
        saveAsDlg* sd = new saveAsDlg(this, "Save As", true);
        sd->setDepth(draw->dv->getDepth(), draw->dv->isGray());

        if(doc->name().isEmpty()){
            sd->setName("New Image");
            sd->setType("PNG");
        } else {
            sd->setName(doc->name());
            sd->setMimeType(doc->type());
        }
        int result = sd->exec();
        if(result == QDialog::Accepted){
            ext = sd->extension();
            format = sd->format();
            QString docname = sd->getName();

            QString dNum = "";
            int num = 0;
            QString fileName = doc->file();
            int pos = fileName.findRev('.');
            if(pos > 0){
                fileName.truncate(pos);
            }
            fileName.replace(QRegExp(doc->type()), sd->getMimeType());
            DocLnk dl;
            dl.setName(docname);
            dl.setType(sd->getMimeType());
            dl.setFile(fileName + dNum + ext);
            FileManager fm;
            while(fm.exists(dl)){
                num++;
                dNum.sprintf("%02d", num);
                dl.setName(docname + dNum);
                dl.setFile(fileName + dNum + ext);
            }
            if(saveAs){
                delete doc;
                doc = new DocLnk();
            }
            doc->setName(docname + dNum);
            doc->setType(sd->getMimeType());
            doc->setFile(fileName + dNum + ext);
            fm.saveFile(*doc, "");
            updateCaption();
            quality = sd->getQuality();
        } else {
            return false;
        }
    }

    if(!draw->dv->getPixmap().save(doc->file(), format, quality)){
        if(saveAs){
            QMessageBox::critical( this, tr( "Unable to save" ),
				    tr( "qpPhoto was unable to\n"
					"save your changes.\n"
                    "Unsupported format.\n"
					"Choose another image format\n"
                    "and try again.\n" ));
            doc->removeFiles();
        }
        return false;
    }
    if(!saveAs){
        delete doc;
        doc = 0;
    }
    draw->dv->setEdited( false );
    return true;
}

void qpPhoto::clear(bool askNewSize)
{
    delete doc;
    doc = 0;
    if(askNewSize)
        draw->dv->init();
    else
        draw->dv->init(0,0);
}


void qpPhoto::updateCaption( const QString& name )
{
    if ( !doc )
	    setCaption( tr("qpPhoto") );
    else {
        QString s = name;
        if ( s.isNull() )
            s = doc->name();
        if ( s.isEmpty() )
            s = tr( "Unnamed" );
        int sep = s.findRev( '/' );
        if ( sep >= 0 )
            s = s.mid( sep+1 );
        int level = draw->dv->getZoomIndex();
        switch(level){
            case 0:
                s += tr(" 25%");
                break;
            case 1:
                s += tr(" 50%");
                break;
            case 2:
                s += tr(" 75%");
                break;
            case 3:
                s += tr(" 100%");
                break;
            case 4:
                s += tr(" 200%");
                break;
            case 5:
                s += tr(" 400%");
                break;
            case 6:
                s += tr(" 800%");
                break;
        }
        setCaption( s + " - " + tr("qpPhoto") );
    }
}

