/**********************************************************************
** Copyright (C) 2002 Edgar Hindemith.  All rights reserved.
**
** email: ed@dom.de
**
** This file is part of the qpPhoto picture editor.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/
#ifndef QPPHOTO_H
#define QPPHOTO_H


#include <qpe/filemanager.h>

#include <qmainwindow.h>

class QWidgetStack;
class QToolButton;
class QPopupMenu;
class QToolBar;
class QAction;
class FileSelector;
class Draw;

class qpPhoto : public QMainWindow
{
    Q_OBJECT

public:
    qpPhoto( QWidget *parent = 0, const char *name = 0, WFlags f = 0 );
    ~qpPhoto();

    void openFile( const QString & );

protected:
    void closeEvent( QCloseEvent *e );
    bool readUndoConfig();
    void writeUndoConfig(bool enable);

public slots:
    void zoomStateChanged(int);
    void undoStateChanged(int);
    void maskStateChanged();
    void clipStateChanged();
    void colorStateChanged(int, bool);
    void zoom25();
    void zoom50();
    void zoom75();
    void zoom100();
    void zoom200();
    void zoom400();
    void zoom800();

private slots:
    void setDocument(const QString&);

    void fileNew();
    void fileRevert();
    void fileOpen();
    void fileSave();
    void fileSaveAs();

    void about();
    void info();
    void enableUndo(bool);

    void accept();

    void newFile( const DocLnk & );
    void openFile( const DocLnk & );
    void closeFileSelector();

private:
    bool save(bool saveAs = false);
    void clear(bool askNewSize=false);
    void updateCaption( const QString &name=QString::null );

private:
    QWidgetStack *stack;
    FileSelector *fileSelector;
    QToolBar *menu, *editBar;
    DocLnk *doc;
    Draw *draw;
    bool bFromDocView;
    QPopupMenu *mnZoom;
    int z25, z50, z75, z100, z200, z400, z800;
    QAction *ua, *ra, *eua, *cta, *cpa, *pa, *hma, *ima, *cma;
    QAction *zia, *zoa, *cvm, *cvg, *cvi, *cvr, *fbl, *fsh, *fem;
};


#endif
