/**********************************************************************
** Copyright (C) 2002 Edgar Hindemith.  All rights reserved.
**
** email: ed@dom.de
**
** This file is part of the qpPhoto picture editor.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/
#include <qlineedit.h>
#include <qcombobox.h>
#include <qslider.h>
#include "saveAsDlg.h"

static const char* formatJPG={"JPEG"};
static const char* formatGIF={"GIF"};
static const char* formatPNG={"PNG"};
static const char* formatXPM={"XPM"};

static const char* extJPG={".jpg"};
static const char* extGIF={".gif"};
static const char* extPNG={".png"};
static const char* extXPM={".xpm"};

saveAsDlg::saveAsDlg(QWidget* parent, const char* name, bool modal, WFlags fl)
:SaveAs(parent, name, modal, fl)
{
    type = "";
    fileType->clear();
    fileType->insertItem("GIF", 0);
    fileType->insertItem("JPG", 1);
    fileType->insertItem("PNG", 2);
    fileType->insertItem("XPM", 3);
    setQuality(100);
    connect( fileName, SIGNAL(textChanged(const QString &)), this, SLOT(nameChanged(const QString &)) );
    connect( fileType, SIGNAL(activated(const QString &)), this, SLOT(typeChanged(const QString &)) );
    connect( fileQuality, SIGNAL(valueChanged(int)), this, SLOT(qualityChanged(int)) );
}

saveAsDlg::~saveAsDlg()
{
}

void saveAsDlg::setName(const QString& n)
{
    if(n != name){
        name = n;
        fileName->setText(n);
    }
}

bool saveAsDlg::setType(const QString& t)
{
    if(t != type){
        fileQuality->setEnabled(true);
        if(t == "GIF"){
            if(depth > 0 && depth <= 8){
                type = t;
                mimeType = "image/gif";
                fileType->setCurrentItem(0);
                return true;
            }
        } else if(t == "JPG"){
            if((depth == 8 && gray) || depth == 24 || depth == 32){
                type = t;
                mimeType = "image/jpeg";
                if(depth == 8 && gray){
                    fileType->setCurrentItem(1);
                } else {
                    fileType->setCurrentItem(0);
                }
                return true;
            }
        } else if(t == "PNG"){
            type = t;
            mimeType = "image/png";
            if(depth == 8 && gray){
                fileType->setCurrentItem(2);
            } else {
                fileType->setCurrentItem(1);
            }
            return true;
        } else if(t == "XPM"){
            if(depth > 0 && depth <= 8){
                type = t;
                mimeType = "image/x-xpixmap";
                if(depth == 8 && gray){
                    fileType->setCurrentItem(3);
                } else {
                    fileType->setCurrentItem(2);
                }
                fileQuality->setEnabled(false);
                return true;
            }
        }
        return false;
    }
    return true;
}

bool saveAsDlg::setMimeType(const QString& t)
{
    if(t == "image/gif"){
        return setType("GIF");
    } else if(t == "image/jpeg"){
        return setType("JPG");
    } else if(t == "image/png"){
        return setType("PNG");
    } else if(t == "image/x-xpixmap"){
        return setType("XPM");
    }
    return false;
}

const char* saveAsDlg::format()
{
    if(type == "GIF"){
        return formatGIF;
    } else if(type == "JPG"){
        return formatJPG;
    } else if(type == "PNG"){
        return formatPNG;
    } else if(type == "XPM"){
        return formatXPM;
    }
    return "";
}

const char* saveAsDlg::extension()
{
    if(type == "GIF"){
        return extGIF;
    } else if(type == "JPG"){
        return extJPG;
    } else if(type == "PNG"){
        return extPNG;
    } else if(type == "XPM"){
        return extXPM;
    }
    return "";
}

void saveAsDlg::setDepth(int d, bool g)
{
    if(d != depth || g != gray){
        depth = d;
        gray = g;
        type = "";
        fileType->clear();
        if((d > 0 && d < 8) || (d == 8 && !g)){
            fileType->insertItem("GIF", 0);
            fileType->insertItem("PNG", 1);
            fileType->insertItem("XPM", 2);
        }
        if(d == 8 && g){
            fileType->insertItem("GIF", 0);
            fileType->insertItem("JPG", 1);
            fileType->insertItem("PNG", 2);
            fileType->insertItem("XPM", 3);
        }
        if(d == 24 || d == 32){
            fileType->insertItem("JPG", 0);
            fileType->insertItem("PNG", 1);
        }
    }
}

void saveAsDlg::setQuality(int q)
{
    if(q != quality){
        quality = q;
        fileQuality->setValue(q);
    }
}

void saveAsDlg::nameChanged(const QString& n)
{
    if(n != name){
        setName(n);
    }
}

void saveAsDlg::typeChanged(const QString& t)
{
    if(t != type){
        setType(t);
    }
}

void saveAsDlg::qualityChanged(int q)
{
    if(q != quality){
        setQuality(q);
    }
}

